# -*- coding: utf-8 -*-
'''
Interface to SMBIOS/DMI

(Parsing through dmidecode)

External References
-------------------
| `Desktop Management Interface (DMI) <http://www.dmtf.org/standards/dmi>`_
| `System Management BIOS <http://www.dmtf.org/standards/smbios>`_
| `DMIdecode <http://www.nongnu.org/dmidecode/>`_

'''
# Import python libs
from __future__ import absolute_import
import logging
import uuid
import re

# Import salt libs
# import salt.log
import salt.utils

# Solve the Chicken and egg problem where grains need to run before any
# of the modules are loaded and are generally available for any usage.
import salt.modules.cmdmod

from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
from salt.ext.six.moves import zip  # pylint: disable=import-error,redefined-builtin

log = logging.getLogger(__name__)

DMIDECODER = salt.utils.which_bin(['dmidecode', 'smbios'])


def __virtual__():
    '''
    Only work when dmidecode is installed.
    '''
    if DMIDECODER is None:
        log.debug('SMBIOS: neither dmidecode nor smbios found!')
        return (False, 'The smbios execution module failed to load: neither dmidecode nor smbios in the path.')
    else:
        return True


def get(string, clean=True):
    '''
    Get an individual DMI string from SMBIOS info

    string
        The string to fetch. DMIdecode supports:
          - ``bios-vendor``
          - ``bios-version``
          - ``bios-release-date``
          - ``system-manufacturer``
          - ``system-product-name``
          - ``system-version``
          - ``system-serial-number``
          - ``system-uuid``
          - ``baseboard-manufacturer``
          - ``baseboard-product-name``
          - ``baseboard-version``
          - ``baseboard-serial-number``
          - ``baseboard-asset-tag``
          - ``chassis-manufacturer``
          - ``chassis-type``
          - ``chassis-version``
          - ``chassis-serial-number``
          - ``chassis-asset-tag``
          - ``processor-family``
          - ``processor-manufacturer``
          - ``processor-version``
          - ``processor-frequency``

    clean
      | Don't return well-known false information
      | (invalid UUID's, serial 000000000's, etcetera)
      | Defaults to ``True``

    CLI Example:

    .. code-block:: bash

        salt '*' smbios.get system-uuid clean=False
    '''

    val = _dmidecoder('-s {0}'.format(string)).strip()

    # Sometimes dmidecode delivers comments in strings.
    # Don't.
    val = '\n'.join([v for v in val.split('\n') if not v.startswith('#')])

    # handle missing /dev/mem
    if val.startswith('/dev/mem'):
        return None

    if not clean or _dmi_isclean(string, val):
        return val


def records(rec_type=None, fields=None, clean=True):
    '''
    Return DMI records from SMBIOS

    type
        Return only records of type(s)
        The SMBIOS specification defines the following DMI types:

        ====  ======================================
        Type  Information
        ====  ======================================
         0    BIOS
         1    System
         2    Baseboard
         3    Chassis
         4    Processor
         5    Memory Controller
         6    Memory Module
         7    Cache
         8    Port Connector
         9    System Slots
        10    On Board Devices
        11    OEM Strings
        12    System Configuration Options
        13    BIOS Language
        14    Group Associations
        15    System Event Log
        16    Physical Memory Array
        17    Memory Device
        18    32-bit Memory Error
        19    Memory Array Mapped Address
        20    Memory Device Mapped Address
        21    Built-in Pointing Device
        22    Portable Battery
        23    System Reset
        24    Hardware Security
        25    System Power Controls
        26    Voltage Probe
        27    Cooling Device
        28    Temperature Probe
        29    Electrical Current Probe
        30    Out-of-band Remote Access
        31    Boot Integrity Services
        32    System Boot
        33    64-bit Memory Error
        34    Management Device
        35    Management Device Component
        36    Management Device Threshold Data
        37    Memory Channel
        38    IPMI Device
        39    Power Supply
        40    Additional Information
        41    Onboard Devices Extended Information
        42    Management Controller Host Interface
        ====  ======================================

    clean
      | Don't return well-known false information
      | (invalid UUID's, serial 000000000's, etcetera)
      | Defaults to ``True``

    CLI Example:

    .. code-block:: bash

        salt '*' smbios.records clean=False
        salt '*' smbios.records 14
        salt '*' smbios.records 4 core_count,thread_count,current_speed

    '''
    if rec_type is None:
        smbios = _dmi_parse(_dmidecoder(), clean, fields)
    else:
        smbios = _dmi_parse(_dmidecoder('-t {0}'.format(rec_type)), clean, fields)

    return smbios


def _dmi_parse(data, clean=True, fields=None):
    '''
    Structurize DMI records into a nice list
    Optionally trash bogus entries and filter output
    '''
    dmi = []

    # Detect & split Handle records
    dmi_split = re.compile('(handle [0-9]x[0-9a-f]+[^\n]+)\n', re.MULTILINE+re.IGNORECASE)
    dmi_raw = iter(re.split(dmi_split, data)[1:])
    for handle, dmi_raw in zip(dmi_raw, dmi_raw):
        handle, htype = [hline.split()[-1] for hline in handle.split(',')][0:2]
        dmi_raw = dmi_raw.split('\n')
        # log.debug('{0} record contains {1}'.format(handle, dmi_raw))
        log.debug('Parsing handle {0}'.format(handle))

        # The first line of a handle is a description of the type
        record = {
            'handle':      handle,
            'description': dmi_raw.pop(0).strip(),
            'type':        int(htype)
        }

        if not len(dmi_raw):
            # empty record
            if not clean:
                dmi.append(record)
            continue

        # log.debug('{0} record contains {1}'.format(record, dmi_raw))
        dmi_data = _dmi_data(dmi_raw, clean, fields)
        if len(dmi_data):
            record['data'] = dmi_data
            dmi.append(record)
        elif not clean:
            dmi.append(record)

    return dmi


def _dmi_data(dmi_raw, clean, fields):
    '''
    Parse the raw DMIdecode output of a single handle
    into a nice dict
    '''
    dmi_data = {}

    key = None
    key_data = [None, []]
    for line in dmi_raw:
        if re.match(r'\t[^\s]+', line):
            # Finish previous key
            if key is not None:
                # log.debug('Evaluating DMI key {0}: {1}'.format(key, key_data))
                value, vlist = key_data
                if len(vlist):
                    if value is not None:
                        # On the rare occasion
                        # (I counted 1 on all systems we have)
                        # that there's both a value <and> a list
                        # just insert the value on top of the list
                        vlist.insert(0, value)
                    dmi_data[key] = vlist
                elif value is not None:
                    dmi_data[key] = value

            # Family: Core i5
            # Keyboard Password Status: Not Implemented
            key, val = line.split(':', 1)
            key = key.strip().lower().replace(' ', '_')
            if (clean and key == 'header_and_data') \
                    or (fields and key not in fields):
                key = None
                continue
            else:
                key_data = [_dmi_cast(key, val.strip(), clean), []]
        elif key is None:
            continue
        elif re.match(r'\t\t[^\s]+', line):
            # Installable Languages: 1
            #        en-US
            # Characteristics:
            #        PCI is supported
            #        PNP is supported
            val = _dmi_cast(key, line.strip(), clean)
            if val is not None:
                # log.debug('DMI key {0} gained list item {1}'.format(key, val))
                key_data[1].append(val)

    return dmi_data


def _dmi_cast(key, val, clean=True):
    '''
    Simple caster thingy for trying to fish out at least ints & lists from strings
    '''
    if clean and not _dmi_isclean(key, val):
        return
    elif not re.match(r'serial|part|asset|product', key, flags=re.IGNORECASE):
        if ',' in val:
            val = [el.strip() for el in val.split(',')]
        else:
            try:
                val = int(val)
            # pylint: disable=bare-except
            except:
                pass

    return val


def _dmi_isclean(key, val):
    '''
    Clean out well-known bogus values
    '''
    if val is None or not len(val) or re.match('none', val, flags=re.IGNORECASE):
        # log.debug('DMI {0} value {1} seems invalid or empty'.format(key, val))
        return False
    elif 'uuid' in key:
        # Try each version (1-5) of RFC4122 to check if it's actually a UUID
        for uuidver in range(1, 5):
            try:
                uuid.UUID(val, version=uuidver)
                return True
            except ValueError:
                continue
        log.trace('DMI {0} value {1} is an invalid UUID'.format(key, val.replace('\n', ' ')))
        return False
    elif re.search('serial|part|version', key):
        # 'To be filled by O.E.M.
        # 'Not applicable' etc.
        # 'Not specified' etc.
        # 0000000, 1234667 etc.
        # begone!
        return not re.match(r'^[0]+$', val) \
                and not re.match(r'[0]?1234567[8]?[9]?[0]?', val) \
                and not re.search(r'sernum|part[_-]?number|specified|filled|applicable', val, flags=re.IGNORECASE)
    elif re.search('asset|manufacturer', key):
        # AssetTag0. Manufacturer04. Begone.
        return not re.search(r'manufacturer|to be filled|available|asset|^no(ne|t)', val, flags=re.IGNORECASE)
    else:
        # map unspecified, undefined, unknown & whatever to None
        return not re.search(r'to be filled', val, flags=re.IGNORECASE) \
            and not re.search(r'un(known|specified)|no(t|ne)? (asset|provided|defined|available|present|specified)',
                              val, flags=re.IGNORECASE)


def _dmidecoder(args=None):
    '''
    Call DMIdecode
    '''
    if args is None:
        return salt.modules.cmdmod._run_quiet(DMIDECODER)
    else:
        return salt.modules.cmdmod._run_quiet('{0} {1}'.format(DMIDECODER, args))



# -*- coding: utf-8 -*-
'''
tests for host state
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing libs
from salttesting.helpers import ensure_in_syspath
ensure_in_syspath('../../')

# Import salt libs
import integration


class HandleErrorTest(integration.ModuleCase):
    '''
    Validate that ordering works correctly
    '''
    def test_handle_error(self):
        '''
        Test how an error can be recovered
        '''
        # without sync_states, the custom state may not be installed
        # (resulting in :
        # State salttest.hello found in sls issue-... is unavailable
        ret = self.run_function('state.sls', ['issue-9983-handleerror'])
        self.assertTrue(
            'An exception occurred in this state: Traceback'
            in ret[[a for a in ret][0]]['comment'])


if __name__ == '__main__':
    from integration import run_tests
    run_tests(HandleErrorTest)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

# Import Salt Libs
from salt.states import win_license as license

# Import Salt Testing Libs
from salttesting import TestCase
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch
)

ensure_in_syspath('../../')

license.__salt__ = {}


class LicenseTestCase(TestCase):

    def test_activate(self):
        '''
            Test activating the given product key
        '''
        expected = {
            'changes': {},
            'comment': 'Windows is now activated.',
            'name': 'AAAAA-AAAAA-AAAAA-AAAA-AAAAA-ABCDE',
            'result': True
        }

        info = {
            'description': 'Prof',
            'licensed': False,
            'name': 'Win7',
            'partial_key': 'XXXXX'
        }

        info_mock = MagicMock(return_value=info)
        install_mock = MagicMock(return_value='Installed successfully')
        activate_mock = MagicMock(return_value='Activated successfully')
        with patch.dict(license.__salt__, {'license.info': info_mock,
                                           'license.install': install_mock,
                                           'license.activate': activate_mock}):
            out = license.activate('AAAAA-AAAAA-AAAAA-AAAA-AAAAA-ABCDE')
            info_mock.assert_called_once_with()
            install_mock.assert_called_once_with('AAAAA-AAAAA-AAAAA-AAAA-AAAAA-ABCDE')
            activate_mock.assert_called_once_with()
            self.assertEqual(out, expected)

    def test_installed_not_activated(self):
        '''
            Test activating the given product key when the key is installed but not activated
        '''
        expected = {
            'changes': {},
            'comment': 'Windows is now activated.',
            'name': 'AAAAA-AAAAA-AAAAA-AAAA-AAAAA-ABCDE',
            'result': True
        }

        info = {
            'description': 'Prof',
            'licensed': False,
            'name': 'Win7',
            'partial_key': 'ABCDE'
        }

        info_mock = MagicMock(return_value=info)
        install_mock = MagicMock(return_value='Installed successfully')
        activate_mock = MagicMock(return_value='Activated successfully')
        with patch.dict(license.__salt__, {'license.info': info_mock,
                                           'license.install': install_mock,
                                           'license.activate': activate_mock}):
            out = license.activate('AAAAA-AAAAA-AAAAA-AAAA-AAAAA-ABCDE')
            info_mock.assert_called_once_with()
            assert not install_mock.called
            activate_mock.assert_called_once_with()
            self.assertEqual(out, expected)

    def test_installed_activated(self):
        '''
            Test activating the given product key when its already activated
        '''
        expected = {
            'changes': {},
            'comment': 'Windows is already activated.',
            'name': 'AAAAA-AAAAA-AAAAA-AAAA-AAAAA-ABCDE',
            'result': True
        }

        info = {
            'description': 'Prof',
            'licensed': True,
            'name': 'Win7',
            'partial_key': 'ABCDE'
        }

        info_mock = MagicMock(return_value=info)
        install_mock = MagicMock(return_value='Installed successfully')
        activate_mock = MagicMock(return_value='Activated successfully')
        with patch.dict(license.__salt__, {'license.info': info_mock,
                                           'license.install': install_mock,
                                           'license.activate': activate_mock}):
            out = license.activate('AAAAA-AAAAA-AAAAA-AAAA-AAAAA-ABCDE')
            info_mock.assert_called_once_with()
            assert not install_mock.called
            assert not activate_mock.called
            self.assertEqual(out, expected)

    def test_installed_install_fail(self):
        '''
            Test activating the given product key when the install fails
        '''
        expected = {
            'changes': {},
            'comment': 'Unable to install the given product key is it valid?',
            'name': 'AAAAA-AAAAA-AAAAA-AAAA-AAAAA-ABCDE',
            'result': False
        }

        info = {
            'description': 'Prof',
            'licensed': False,
            'name': 'Win7',
            'partial_key': '12345'
        }

        info_mock = MagicMock(return_value=info)
        install_mock = MagicMock(return_value='Failed')
        activate_mock = MagicMock()
        with patch.dict(license.__salt__, {'license.info': info_mock,
                                           'license.install': install_mock,
                                           'license.activate': activate_mock}):
            out = license.activate('AAAAA-AAAAA-AAAAA-AAAA-AAAAA-ABCDE')
            info_mock.assert_called_once_with()
            install_mock.assert_called_once_with('AAAAA-AAAAA-AAAAA-AAAA-AAAAA-ABCDE')
            assert not activate_mock.called
            self.assertEqual(out, expected)

    def test_installed_activate_fail(self):
        '''
            Test activating the given product key when the install fails
        '''
        expected = {
            'changes': {},
            'comment': 'Unable to activate the given product key.',
            'name': 'AAAAA-AAAAA-AAAAA-AAAA-AAAAA-ABCDE',
            'result': False
        }

        info = {
            'description': 'Prof',
            'licensed': False,
            'name': 'Win7',
            'partial_key': 'ABCDE'
        }

        info_mock = MagicMock(return_value=info)
        install_mock = MagicMock()
        activate_mock = MagicMock(return_value='Failed to activate')
        with patch.dict(license.__salt__, {'license.info': info_mock,
                                           'license.install': install_mock,
                                           'license.activate': activate_mock}):
            out = license.activate('AAAAA-AAAAA-AAAAA-AAAA-AAAAA-ABCDE')
            info_mock.assert_called_once_with()
            assert not install_mock.called
            activate_mock.assert_called_once_with()
            self.assertEqual(out, expected)

if __name__ == '__main__':
    from integration import run_tests
    run_tests(LicenseTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import
import string
import random

# Import Salt Testing libs
from salttesting.helpers import ensure_in_syspath, destructiveTest
ensure_in_syspath('../../')

# Import salt libs
from salt.ext.six.moves import range
import integration
import salt.utils


class GroupModuleTest(integration.ModuleCase):
    '''
    Validate the linux group system module
    '''

    def __init__(self, arg):
        super(self.__class__, self).__init__(arg)
        self._user = self.__random_string()
        self._user1 = self.__random_string()
        self._no_user = self.__random_string()
        self._group = self.__random_string()
        self._no_group = self.__random_string()
        self._gid = 64989
        self._new_gid = 64998

    def setUp(self):
        '''
        Get current settings
        '''
        super(GroupModuleTest, self).setUp()
        os_grain = self.run_function('grains.item', ['kernel'])
        if os_grain['kernel'] not in 'Linux':
            self.skipTest(
                'Test not applicable to \'{kernel}\' kernel'.format(
                    **os_grain
                )
            )
        if salt.utils.get_uid(salt.utils.get_user()) != 0:
            self.skipTest('Tests requires root')

    @destructiveTest
    def tearDown(self):
        '''
        Reset to original settings
        '''
        self.run_function('user.delete', [self._user])
        self.run_function('user.delete', [self._user1])
        self.run_function('group.delete', [self._group])

    def __random_string(self, size=6):
        '''
        Generates a random names
        '''
        return 'tg-' + ''.join(
            random.choice(string.ascii_lowercase + string.digits)
            for x in range(size)
        )

    @destructiveTest
    def test_add(self):
        '''
        Test the add group function
        '''
        #add a new group
        self.assertTrue(self.run_function('group.add', [self._group, self._gid]))
        group_info = self.run_function('group.info', [self._group])
        self.assertEqual(group_info['name'], self._group)
        self.assertEqual(group_info['gid'], self._gid)
        #try adding the group again
        self.assertFalse(self.run_function('group.add', [self._group, self._gid]))

    @destructiveTest
    def test_delete(self):
        '''
        Test the delete group function
        '''
        self.assertTrue(self.run_function('group.add', [self._group]))

        #correct functionality
        self.assertTrue(self.run_function('group.delete', [self._group]))

        #group does not exist
        self.assertFalse(self.run_function('group.delete', [self._no_group]))

    @destructiveTest
    def test_info(self):
        '''
        Test the info group function
        '''
        self.run_function('group.add', [self._group, self._gid])
        self.run_function('user.add', [self._user])
        self.run_function('group.adduser', [self._group, self._user])
        group_info = self.run_function('group.info', [self._group])

        self.assertEqual(group_info['name'], self._group)
        self.assertEqual(group_info['gid'], self._gid)
        self.assertIn(self._user, group_info['members'])

    @destructiveTest
    def test_chgid(self):
        '''
        Test the change gid function
        '''
        self.run_function('group.add', [self._group, self._gid])
        self.assertTrue(self.run_function('group.chgid', [self._group, self._new_gid]))
        group_info = self.run_function('group.info', [self._group])
        self.assertEqual(group_info['gid'], self._new_gid)

    @destructiveTest
    def test_adduser(self):
        '''
        Test the add user to group function
        '''
        self.run_function('group.add', [self._group, self._gid])
        self.run_function('user.add', [self._user])
        self.assertTrue(self.run_function('group.adduser', [self._group, self._user]))
        group_info = self.run_function('group.info', [self._group])
        self.assertIn(self._user, group_info['members'])
        #try add a non existing user
        self.assertFalse(self.run_function('group.adduser', [self._group, self._no_user]))
        #try add a user to non existing group
        self.assertFalse(self.run_function('group.adduser', [self._no_group, self._user]))
        #try add a non existing user to a non existing group
        self.assertFalse(self.run_function('group.adduser', [self._no_group, self._no_user]))

    @destructiveTest
    def test_deluser(self):
        '''
        Test the delete user from group function
        '''
        self.run_function('group.add', [self._group, self._gid])
        self.run_function('user.add', [self._user])
        self.run_function('group.adduser', [self._group, self._user])
        self.assertTrue(self.run_function('group.deluser', [self._group, self._user]))
        group_info = self.run_function('group.info', [self._group])
        self.assertNotIn(self._user, group_info['members'])

    @destructiveTest
    def test_members(self):
        '''
        Test the members function
        '''
        self.run_function('group.add', [self._group, self._gid])
        self.run_function('user.add', [self._user])
        self.run_function('user.add', [self._user1])
        m = '{0},{1}'.format(self._user, self._user1)
        self.assertTrue(self.run_function('group.members', [self._group, m]))
        group_info = self.run_function('group.info', [self._group])
        self.assertIn(self._user, group_info['members'])
        self.assertIn(self._user1, group_info['members'])

    @destructiveTest
    def test_getent(self):
        '''
        Test the getent function
        '''
        self.run_function('group.add', [self._group, self._gid])
        self.run_function('user.add', [self._user])
        self.run_function('group.adduser', [self._group, self._user])
        ginfo = self.run_function('user.getent')
        self.assertIn(self._group, str(ginfo))
        self.assertIn(self._user, str(ginfo))
        self.assertNotIn(self._no_group, str(ginfo))
        self.assertNotIn(self._no_user, str(ginfo))


if __name__ == '__main__':
    from integration import run_tests
    run_tests(GroupModuleTest)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import MagicMock, patch

ensure_in_syspath('../../')

# Import Salt Libs
from salt.exceptions import CommandExecutionError
from salt.modules import mac_xattr as xattr
import salt.utils.mac_utils

xattr.__salt__ = {}


class XAttrTestCase(TestCase):

    @patch('salt.utils.mac_utils.execute_return_result',
           MagicMock(return_value='spongebob\nsquidward'))
    def test_list(self):
        '''
        Test xattr.list
        '''
        expected = {'spongebob': 'squarepants',
                    'squidward': 'patrick'}
        with patch.object(xattr, 'read', MagicMock(side_effect=['squarepants',
                                                                'patrick'])):
            self.assertEqual(xattr.list_('path/to/file'), expected)

    @patch('salt.utils.mac_utils.execute_return_result',
           MagicMock(side_effect=CommandExecutionError('No such file')))
    def test_list_missing(self):
        '''
        Test listing attributes of a missing file
        '''
        self.assertRaises(CommandExecutionError, xattr.list_, '/path/to/file')

    @patch('salt.utils.mac_utils.execute_return_result',
           MagicMock(return_value='expected results'))
    def test_read(self):
        '''
        Test reading a specific attribute from a file
        '''
        self.assertEqual(xattr.read('/path/to/file', 'com.attr'),
                         'expected results')

    def test_read_hex(self):
        '''
        Test reading a specific attribute from a file
        '''
        with patch.object(salt.utils.mac_utils, 'execute_return_result',
                          MagicMock(return_value='expected results')) as mock:
            self.assertEqual(
                xattr.read('/path/to/file', 'com.attr', **{'hex': True}),
                'expected results'
            )
            mock.assert_called_once_with(
                ['xattr', '-p', '-x', 'com.attr', '/path/to/file'])

    @patch('salt.utils.mac_utils.execute_return_result',
           MagicMock(side_effect=CommandExecutionError('No such file')))
    def test_read_missing(self):
        '''
        Test reading a specific attribute from a file
        '''
        self.assertRaises(CommandExecutionError,
                          xattr.read,
                          '/path/to/file',
                          'attribute')

    @patch('salt.utils.mac_utils.execute_return_success',
           MagicMock(return_value=True))
    def test_write(self):
        '''
        Test writing a specific attribute to a file
        '''
        mock_cmd = MagicMock(return_value='squarepants')
        with patch.object(xattr, 'read', mock_cmd):
            self.assertTrue(xattr.write('/path/to/file',
                                        'spongebob',
                                        'squarepants'))

    @patch('salt.utils.mac_utils.execute_return_success',
           MagicMock(side_effect=CommandExecutionError('No such file')))
    def test_write_missing(self):
        '''
        Test writing a specific attribute to a file
        '''
        self.assertRaises(CommandExecutionError,
                          xattr.write,
                          '/path/to/file',
                          'attribute',
                          'value')

    @patch('salt.utils.mac_utils.execute_return_success',
           MagicMock(return_value=True))
    def test_delete(self):
        '''
        Test deleting a specific attribute from a file
        '''
        mock_cmd = MagicMock(return_value={'spongebob': 'squarepants'})
        with patch.object(xattr, 'list_', mock_cmd):
            self.assertTrue(xattr.delete('/path/to/file', 'attribute'))

    @patch('salt.utils.mac_utils.execute_return_success',
           MagicMock(side_effect=CommandExecutionError('No such file')))
    def test_delete_missing(self):
        '''
        Test deleting a specific attribute from a file
        '''
        self.assertRaises(CommandExecutionError,
                          xattr.delete,
                          '/path/to/file',
                          'attribute')

    @patch('salt.utils.mac_utils.execute_return_success',
           MagicMock(return_value=True))
    def test_clear(self):
        '''
        Test clearing all attributes on a file
        '''
        mock_cmd = MagicMock(return_value={})
        with patch.object(xattr, 'list_', mock_cmd):
            self.assertTrue(xattr.clear('/path/to/file'))

    @patch('salt.utils.mac_utils.execute_return_success',
           MagicMock(side_effect=CommandExecutionError('No such file')))
    def test_clear_missing(self):
        '''
        Test clearing all attributes on a file
        '''
        self.assertRaises(CommandExecutionError, xattr.clear, '/path/to/file')


if __name__ == '__main__':
    from integration import run_tests
    run_tests(XAttrTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

# Import Salt Libs
from salt.modules import win_dism as dism

# Import Salt Testing Libs
from salttesting import TestCase
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch
)

ensure_in_syspath('../../')

dism.__salt__ = {}
dism.__grains__ = {}


class WinDismTestCase(TestCase):

    def test_add_capability(self):
        '''
            Test installing a capability with DISM
        '''
        mock = MagicMock()
        with patch.dict(dism.__salt__, {'cmd.run_all': mock}):
            with patch.dict(dism.__grains__, {'osversion': 10}):
                dism.add_capability("test")
                mock.assert_called_once_with(
                    ['DISM', '/Quiet', '/Online', '/Add-Capability',
                     '/CapabilityName:test', '/NoRestart'])

    def test_add_capability_with_extras(self):
        '''
            Test installing a capability with DISM
        '''
        mock = MagicMock()
        with patch.dict(dism.__salt__, {'cmd.run_all': mock}):
            with patch.dict(dism.__grains__, {'osversion': 10}):
                dism.add_capability("test", "life", True)
                mock.assert_called_once_with(
                    ['DISM', '/Quiet', '/Online', '/Add-Capability',
                     '/CapabilityName:test', '/Source:life', '/LimitAccess',
                     '/NoRestart'])

    def test_remove_capability(self):
        '''
            Test uninstalling a capability with DISM
        '''
        mock = MagicMock()
        with patch.dict(dism.__salt__, {'cmd.run_all': mock}):
            with patch.dict(dism.__grains__, {'osversion': 10}):
                dism.remove_capability("test")
                mock.assert_called_once_with(
                    ['DISM', '/Quiet', '/Online', '/Remove-Capability',
                     '/CapabilityName:test', '/NoRestart'])

    def test_get_capabilities(self):
        '''
            Test getting all the capabilities
        '''
        capabilties = "Capability Identity : Capa1\r\n State : Installed\r\n" \
                      "Capability Identity : Capa2\r\n State : Disabled\r\n"

        mock = MagicMock(return_value=capabilties)
        with patch.dict(dism.__salt__, {'cmd.run': mock}):
            with patch.dict(dism.__grains__, {'osversion': 10}):
                out = dism.get_capabilities()
                mock.assert_called_once_with(
                    ['DISM', '/English', '/Online', '/Get-Capabilities'])
                self.assertEqual(out, ['Capa1', 'Capa2'])

    def test_installed_capabilities(self):
        '''
            Test getting all the installed capabilities
        '''
        capabilties = "Capability Identity : Capa1\r\n State : Installed\r\n" \
                      "Capability Identity : Capa2\r\n State : Disabled\r\n"

        mock = MagicMock(return_value=capabilties)
        with patch.dict(dism.__salt__, {'cmd.run': mock}):
            with patch.dict(dism.__grains__, {'osversion': 10}):
                out = dism.installed_capabilities()
                mock.assert_called_once_with(
                    ['DISM', '/English', '/Online', '/Get-Capabilities'])
                self.assertEqual(out, ["Capa1"])

    def test_available_capabilities(self):
        '''
            Test getting all the available capabilities
        '''
        capabilties = "Capability Identity : Capa1\r\n State : Installed\r\n" \
                      "Capability Identity : Capa2\r\n State : Not Present\r\n"

        mock = MagicMock(return_value=capabilties)
        with patch.dict(dism.__salt__, {'cmd.run': mock}):
            with patch.dict(dism.__grains__, {'osversion': 10}):
                out = dism.available_capabilities()
                mock.assert_called_once_with(
                    ['DISM', '/English', '/Online', '/Get-Capabilities'])
                self.assertEqual(out, ["Capa2"])

    def test_add_feature(self):
        '''
            Test installing a feature with DISM
        '''
        mock = MagicMock()
        with patch.dict(dism.__salt__, {'cmd.run_all': mock}):
            dism.add_feature("test")
            mock.assert_called_once_with(
                ['DISM', '/Quiet', '/Online', '/Enable-Feature',
                 '/FeatureName:test', '/NoRestart'])

    def test_add_feature_with_extras(self):
        '''
            Test installing a feature with DISM
        '''
        mock = MagicMock()
        with patch.dict(dism.__salt__, {'cmd.run_all': mock}):
            dism.add_feature('sponge', 'bob', 'C:\\temp', True, True)
            mock.assert_called_once_with(
                ['DISM', '/Quiet', '/Online', '/Enable-Feature',
                 '/FeatureName:sponge', '/PackageName:bob', '/Source:C:\\temp',
                 '/LimitAccess', '/All', '/NoRestart'])

    def test_remove_feature(self):
        '''
            Test uninstalling a capability with DISM
        '''
        mock = MagicMock()
        with patch.dict(dism.__salt__, {'cmd.run_all': mock}):
            dism.remove_feature("test")
            mock.assert_called_once_with(
                ['DISM', '/Quiet', '/Online', '/Disable-Feature',
                 '/FeatureName:test', '/NoRestart'])

    def test_remove_feature_with_extras(self):
        '''
            Test uninstalling a capability with DISM
        '''
        mock = MagicMock()
        with patch.dict(dism.__salt__, {'cmd.run_all': mock}):
            dism.remove_feature('sponge', True)
            mock.assert_called_once_with(
                ['DISM', '/Quiet', '/Online', '/Disable-Feature',
                 '/FeatureName:sponge', '/Remove', '/NoRestart'])

    def test_get_features(self):
        '''
            Test getting all the features
        '''
        features = "Feature Name : Capa1\r\n State : Enabled\r\n" \
                      "Feature Name : Capa2\r\n State : Disabled\r\n"

        mock = MagicMock(return_value=features)
        with patch.dict(dism.__salt__, {'cmd.run': mock}):
            out = dism.get_features()
            mock.assert_called_once_with(['DISM', '/English', '/Online', '/Get-Features'])
            self.assertEqual(out, ['Capa1', 'Capa2'])

    def test_installed_features(self):
        '''
            Test getting all the installed features
        '''
        features = "Feature Name : Capa1\r\n State : Enabled\r\n" \
                      "Feature Name : Capa2\r\n State : Disabled\r\n"

        mock = MagicMock(return_value=features)
        with patch.dict(dism.__salt__, {'cmd.run': mock}):
            out = dism.installed_features()
            mock.assert_called_once_with(['DISM', '/English', '/Online', '/Get-Features'])
            self.assertEqual(out, ["Capa1"])

    def test_available_features(self):
        '''
            Test getting all the available features
        '''
        features = "Feature Name : Capa1\r\n State : Enabled\r\n" \
                   "Feature Name : Capa2\r\n State : Disabled\r\n"

        mock = MagicMock(return_value=features)
        with patch.dict(dism.__salt__, {'cmd.run': mock}):
            out = dism.available_features()
            mock.assert_called_once_with(['DISM', '/English', '/Online', '/Get-Features'])
            self.assertEqual(out, ["Capa2"])

    def test_add_package(self):
        '''
            Test installing a package with DISM
        '''
        mock = MagicMock()
        with patch.dict(dism.__salt__, {'cmd.run_all': mock}):
            dism.add_package("test")
            mock.assert_called_once_with(
                ['DISM', '/Quiet', '/Online', '/Add-Package',
                 '/PackagePath:test', '/NoRestart'])

    def test_add_package_with_extras(self):
        '''
            Test installing a package with DISM
        '''
        mock = MagicMock()
        with patch.dict(dism.__salt__, {'cmd.run_all': mock}):
            dism.add_package('sponge', True, True)
            mock.assert_called_once_with(
                ['DISM', '/Quiet', '/Online', '/Add-Package',
                 '/PackagePath:sponge', '/IgnoreCheck', '/PreventPending',
                 '/NoRestart'])

    def test_remove_package(self):
        '''
            Test uninstalling a package with DISM
        '''
        mock = MagicMock()
        with patch.dict(dism.__salt__, {'cmd.run_all': mock}):
            dism.remove_package("test")
            mock.assert_called_once_with(
                ['DISM', '/Quiet', '/Online', '/Remove-Package', '/NoRestart',
                 '/PackagePath:test'])

    def test_installed_packages(self):
        '''
            Test getting all the installed features
        '''
        features = "Package Identity : Capa1\r\n State : Installed\r\n" \
                   "Package Identity : Capa2\r\n State : Installed\r\n"

        mock = MagicMock(return_value=features)
        with patch.dict(dism.__salt__, {'cmd.run': mock}):
            out = dism.installed_packages()
            mock.assert_called_once_with(['DISM', '/English', '/Online', '/Get-Packages'])
            self.assertEqual(out, ['Capa1', 'Capa2'])

if __name__ == '__main__':
    from integration import run_tests
    run_tests(WinDismTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
virst compatibility module for managing VMs on SmartOS
'''
from __future__ import absolute_import

# Import Python libs
import logging

# Import Salt libs
from salt.exceptions import CommandExecutionError
import salt.utils

log = logging.getLogger(__name__)

# Define the module's virtual name
__virtualname__ = 'virt'


def __virtual__():
    '''
    Provides virt on SmartOS
    '''
    if salt.utils.is_smartos_globalzone() and salt.utils.which('vmadm'):
        return __virtualname__
    return (
        False,
        '{0} module can only be loaded on SmartOS computed nodes'.format(
            __virtualname__
        )
    )


def init(**kwargs):
    '''
    Initialize a new VM

    CLI Example:

    .. code-block:: bash

        salt '*' virt.init image_uuid='...' alias='...' [...]
    '''
    return __salt__['vmadm.create'](**kwargs)


def list_domains():
    '''
    Return a list of virtual machine names on the minion

    CLI Example:

    .. code-block:: bash

        salt '*' virt.list_domains
    '''
    data = __salt__['vmadm.list'](keyed=True)
    vms = []
    vms.append("UUID                                  TYPE  RAM      STATE             ALIAS")
    for vm in data:
        vms.append("{vmuuid}{vmtype}{vmram}{vmstate}{vmalias}".format(
            vmuuid=vm.ljust(38),
            vmtype=data[vm]['type'].ljust(6),
            vmram=data[vm]['ram'].ljust(9),
            vmstate=data[vm]['state'].ljust(18),
            vmalias=data[vm]['alias'],
        ))
    return vms


def list_active_vms():
    '''
    Return a list of uuids for active virtual machine on the minion

    CLI Example:

    .. code-block:: bash

        salt '*' virt.list_active_vms
    '''
    return __salt__['vmadm.list'](search="state='running'", order='uuid')


def list_inactive_vms():
    '''
    Return a list of uuids for inactive virtual machine on the minion

    CLI Example:

    .. code-block:: bash

        salt '*' virt.list_inactive_vms
    '''
    return __salt__['vmadm.list'](search="state='stopped'", order='uuid')


def vm_info(domain):
    '''
    Return a dict with information about the specified VM on this CN

    CLI Example:

    .. code-block:: bash

        salt '*' virt.vm_info <domain>
    '''
    return __salt__['vmadm.get'](domain)


def start(domain):
    '''
    Start a defined domain

    CLI Example:

    .. code-block:: bash

        salt '*' virt.start <domain>
    '''
    if domain in list_active_vms():
        raise CommandExecutionError('The specified vm is already running')

    __salt__['vmadm.start'](domain)

    return domain in list_active_vms()


def shutdown(domain):
    '''
    Send a soft shutdown signal to the named vm

    CLI Example:

    .. code-block:: bash

        salt '*' virt.shutdown <domain>
    '''
    if domain in list_inactive_vms():
        raise CommandExecutionError('The specified vm is already stopped')

    __salt__['vmadm.stop'](domain)

    return domain in list_inactive_vms()


def reboot(domain):
    '''
    Reboot a domain via ACPI request

    CLI Example:

    .. code-block:: bash

        salt '*' virt.reboot <domain>
    '''
    if domain in list_inactive_vms():
        raise CommandExecutionError('The specified vm is stopped')

    __salt__['vmadm.reboot'](domain)

    return domain in list_active_vms()


def stop(domain):
    '''
    Hard power down the virtual machine, this is equivalent to powering off the hardware.

    CLI Example:

    .. code-block:: bash

        salt '*' virt.destroy <domain>
    '''
    if domain in list_inactive_vms():
        raise CommandExecutionError('The specified vm is stopped')

    return __salt__['vmadm.delete'](domain)


def vm_virt_type(domain):
    '''
    Return VM virtualization type : OS or KVM

    CLI Example:

    .. code-block:: bash

        salt '*' virt.vm_virt_type <domain>
    '''
    ret = __salt__['vmadm.lookup'](search="uuid={uuid}".format(uuid=domain), order='type')
    if len(ret) < 1:
        raise CommandExecutionError("We can't determine the type of this VM")

    return ret[0]['type']


def setmem(domain, memory):
    '''
    Change the amount of memory allocated to VM.
    <memory> is to be specified in MB.

    Note for KVM : this would require a restart of the VM.

    CLI Example:

    .. code-block:: bash

        salt '*' virt.setmem <domain> 512
    '''
    vmtype = vm_virt_type(domain)
    if vmtype == 'OS':
        return __salt__['vmadm.update'](vm=domain, max_physical_memory=memory)
    elif vmtype == 'LX':
        return __salt__['vmadm.update'](vm=domain, max_physical_memory=memory)
    elif vmtype == 'KVM':
        log.warning('Changes will be applied after the VM restart.')
        return __salt__['vmadm.update'](vm=domain, ram=memory)
    else:
        raise CommandExecutionError('Unknown VM type')

    return False


def get_macs(domain):
    '''
    Return a list off MAC addresses from the named VM

    CLI Example:

    .. code-block:: bash

        salt '*' virt.get_macs <domain>
    '''
    macs = []
    ret = __salt__['vmadm.lookup'](search="uuid={uuid}".format(uuid=domain), order='nics')
    if len(ret) < 1:
        raise CommandExecutionError('We can\'t find the MAC address of this VM')
    else:
        for nic in ret[0]['nics']:
            macs.append(nic['mac'])
        return macs


# Deprecated aliases
def create(domain):
    '''
    .. deprecated:: 2016.3.0
       Use :py:func:`~salt.modules.virt.start` instead.

    Start a defined domain

    CLI Example:

    .. code-block:: bash

        salt '*' virt.create <domain>
    '''
    salt.utils.warn_until('Nitrogen', 'Use "virt.start" instead.')
    return start(domain)


def destroy(domain):
    '''
    .. deprecated:: 2016.3.0
       Use :py:func:`~salt.modules.virt.stop` instead.

    Power off a defined domain

    CLI Example:

    .. code-block:: bash

        salt '*' virt.destroy <domain>
    '''
    salt.utils.warn_until('Nitrogen', 'Use "virt.stop" instead.')
    return stop(domain)


def list_vms():
    '''
    .. deprecated:: 2016.3.0
       Use :py:func:`~salt.modules.virt.list_domains` instead.

    List all virtual machines.

    CLI Example:

    .. code-block:: bash

        salt '*' virt.list_vms <domain>
    '''
    salt.utils.warn_until('Nitrogen', 'Use "virt.list_domains" instead.')
    return list_domains()



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Logilab <contact@logilab.fr>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import postgres_cluster

postgres_cluster.__opts__ = {}
postgres_cluster.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PostgresClusterTestCase(TestCase):
    '''
    Test cases for salt.states.postgres_cluster
    '''
    # 'present' function tests: 1

    def test_present(self):
        '''
        Test to ensure that the named database is present
        with the specified properties.
        '''
        name = 'main'
        version = '9.4'

        ret = {'name': name,
               'changes': {},
               'result': False,
               'comment': ''}

        mock_t = MagicMock(return_value=True)
        mock_f = MagicMock(return_value=False)
        infos = {'{0}/{1}'.format(version, name): {}}
        mock = MagicMock(return_value=infos)
        with patch.dict(postgres_cluster.__salt__,
                        {'postgres.cluster_list': mock,
                         'postgres.cluster_exists': mock_t,
                         'postgres.cluster_create': mock_t,
                        }):
            comt = ('Cluster {0}/{1} is already present'.format(version, name))
            ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(postgres_cluster.present(version, name), ret)
            infos['{0}/{1}'.format(version, name)]['port'] = 5433
            comt = ('Cluster {0}/{1} has wrong parameters '
                    'which couldn\'t be changed on fly.'.format(version, name))
            ret.update({'comment': comt, 'result': False})
            self.assertDictEqual(postgres_cluster.present(version, name, port=5434), ret)
            infos['{0}/{1}'.format(version, name)]['datadir'] = '/tmp/'
            comt = ('Cluster {0}/{1} has wrong parameters '
                    'which couldn\'t be changed on fly.'.format(version, name))
            ret.update({'comment': comt, 'result': False})
            self.assertDictEqual(postgres_cluster.present(version, name, port=5434), ret)

        with patch.dict(postgres_cluster.__salt__,
                        {'postgres.cluster_list': mock,
                         'postgres.cluster_exists': mock_f,
                         'postgres.cluster_create': mock_t,
                        }):
            comt = 'The cluster {0}/{1} has been created'.format(version, name)
            ret.update({'comment': comt, 'result': True,
                        'changes': {'{0}/{1}'.format(version, name): 'Present'}
                        })
            self.assertDictEqual(postgres_cluster.present(version, name),
                                 ret)
            with patch.dict(postgres_cluster.__opts__, {'test': True}):
                comt = 'Cluster {0}/{1} is set to be created'.format(version, name)
                ret.update({'comment': comt, 'result': None, 'changes': {}})
                self.assertDictEqual(postgres_cluster.present(version, name),
                                     ret)

        with patch.dict(postgres_cluster.__salt__,
                        {'postgres.cluster_list': mock,
                         'postgres.cluster_exists': mock_f,
                         'postgres.cluster_create': mock_f,
                        }):
            comt = 'Failed to create cluster {0}/{1}'.format(version, name)
            ret.update({'comment': comt, 'result': False})
            self.assertDictEqual(postgres_cluster.present(version, name),
                                 ret)

    # 'absent' function tests: 1

    def test_absent(self):
        '''
        Test to ensure that the named database is absent.
        '''
        name = 'main'
        version = '9.4'

        ret = {'name': name,
               'changes': {},
               'result': False,
               'comment': ''}

        mock_t = MagicMock(return_value=True)
        mock = MagicMock(side_effect=[True, True, False])
        with patch.dict(postgres_cluster.__salt__,
                        {'postgres.cluster_exists': mock,
                         'postgres.cluster_remove': mock_t}):
            with patch.dict(postgres_cluster.__opts__, {'test': True}):
                comt = ('Cluster {0}/{1} is set to be removed'.format(version, name))
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(postgres_cluster.absent(version, name), ret)

            with patch.dict(postgres_cluster.__opts__, {'test': False}):
                comt = ('Cluster {0}/{1} has been removed'.format(version, name))
                ret.update({'comment': comt, 'result': True,
                            'changes': {name: 'Absent'}})
                self.assertDictEqual(postgres_cluster.absent(version, name), ret)

                comt = ('Cluster {0}/{1} is not present, so it cannot be removed'
                        .format(version, name))
                ret.update({'comment': comt, 'result': True, 'changes': {}})
                self.assertDictEqual(postgres_cluster.absent(version, name), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(PostgresClusterTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import postgres_database

postgres_database.__opts__ = {}
postgres_database.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PostgresDatabaseTestCase(TestCase):
    '''
    Test cases for salt.states.postgres_database
    '''
    # 'present' function tests: 1

    def test_present(self):
        '''
        Test to ensure that the named database is present
        with the specified properties.
        '''
        name = 'frank'

        ret = {'name': name,
               'changes': {},
               'result': False,
               'comment': ''}

        mock_t = MagicMock(return_value=True)
        mock = MagicMock(return_value={name: {}})
        with patch.dict(postgres_database.__salt__,
                        {'postgres.db_list': mock,
                         'postgres.db_alter': mock_t}):
            comt = ('Database {0} is already present'.format(name))
            ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(postgres_database.present(name), ret)

            comt = ("Database frank has wrong parameters "
                    "which couldn't be changed on fly.")
            ret.update({'comment': comt, 'result': False})
            self.assertDictEqual(postgres_database.present(name, tablespace='A',
                                                           lc_collate=True),
                                 ret)

            with patch.dict(postgres_database.__opts__, {'test': True}):
                comt = ('Database frank exists, '
                        'but parameters need to be changed')
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(postgres_database.present(name,
                                                               tablespace='A'),
                                     ret)

            with patch.dict(postgres_database.__opts__, {'test': False}):
                comt = ('Parameters for database frank have been changed')
                ret.update({'comment': comt, 'result': True,
                            'changes': {name: 'Parameters changed'}})
                self.assertDictEqual(postgres_database.present(name,
                                                               tablespace='A'),
                                     ret)

    # 'absent' function tests: 1

    def test_absent(self):
        '''
        Test to ensure that the named database is absent.
        '''
        name = 'frank'

        ret = {'name': name,
               'changes': {},
               'result': False,
               'comment': ''}

        mock_t = MagicMock(return_value=True)
        mock = MagicMock(side_effect=[True, True, False])
        with patch.dict(postgres_database.__salt__,
                        {'postgres.db_exists': mock,
                         'postgres.db_remove': mock_t}):
            with patch.dict(postgres_database.__opts__, {'test': True}):
                comt = ('Database {0} is set to be removed'.format(name))
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(postgres_database.absent(name), ret)

            with patch.dict(postgres_database.__opts__, {'test': False}):
                comt = ('Database {0} has been removed'.format(name))
                ret.update({'comment': comt, 'result': True,
                            'changes': {name: 'Absent'}})
                self.assertDictEqual(postgres_database.absent(name), ret)

                comt = ('Database {0} is not present, so it cannot be removed'
                        .format(name))
                ret.update({'comment': comt, 'result': True, 'changes': {}})
                self.assertDictEqual(postgres_database.absent(name), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(PostgresDatabaseTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
NAPALM Grains
=============

:codeauthor: Mircea Ulinic <mircea@cloudflare.com>
:maturity:   new
:depends:    napalm
:platform:   unix

Dependencies
------------

- :mod:`NAPALM proxy module <salt.proxies.napalm>`

.. versionadded:: 2016.11.0
'''

from __future__ import absolute_import

import logging
log = logging.getLogger(__name__)

# Salt lib
import salt.utils

# ----------------------------------------------------------------------------------------------------------------------
# grains properties
# ----------------------------------------------------------------------------------------------------------------------

__virtualname__ = 'napalm'
__proxyenabled__ = ['napalm']

# ----------------------------------------------------------------------------------------------------------------------
# global variables
# ----------------------------------------------------------------------------------------------------------------------

GRAINS_CACHE = {}

# ----------------------------------------------------------------------------------------------------------------------
# property functions
# ----------------------------------------------------------------------------------------------------------------------


def __virtual__():
    try:
        if salt.utils.is_proxy() and __opts__['proxy']['proxytype'] == 'napalm':
            return __virtualname__
    except KeyError:
        pass

    return False

# ----------------------------------------------------------------------------------------------------------------------
# helpers
# ----------------------------------------------------------------------------------------------------------------------


def _retrieve_grains(proxy):
    '''
    Retrieves the grains from the network device if not cached already.
    '''

    global GRAINS_CACHE

    if not GRAINS_CACHE:
        GRAINS_CACHE = proxy['napalm.grains']()

    return GRAINS_CACHE


def _get_grain(proxy, name):
    '''
    Retrieves the grain value from the cached dictionary.
    '''
    grains = _retrieve_grains(proxy)
    if grains.get('result', False) and grains.get('out', {}):
        return grains.get('out').get(name)

# ----------------------------------------------------------------------------------------------------------------------
# actual grains
# ----------------------------------------------------------------------------------------------------------------------


def getos():
    '''
    Returns the Operating System name running on the network device.

    Example: junos, iosxr, eos, ios etc.

    CLI Example - select all network devices running JunOS:

    .. code-block:: bash

        salt -G 'os:junos' test.ping
    '''
    # we have this in the pillar
    return {'os': __pillar__.get('proxy', {}).get('driver', '')}


def version(proxy):
    '''
    Returns the OS version.

    Example: 13.3R6.5, 6.0.2 etc.

    CLI Example - select all network devices running JunOS 13.3R6.5 and return the model:

    .. code-block:: bash

        salt -G 'os:junos and version:13.3R6.5' grains.get model

    Output:

    .. code-block:: yaml

        edge01.bjm01:
            MX2000
        edge01.sjc01:
            MX960
        edge01.mrs01:
            MX480
        edge01.muc01:
            MX240
    '''
    if proxy:
        return {'version': _get_grain(proxy, 'os_version')}


def model(proxy):
    '''
    Returns the network device chassis model.

    Example: MX480, ASR-9904-AC etc.

    CLI Example - select all Juniper MX480 routers and execute traceroute to 8.8.8.8:

    .. code-block:: bash

        salt -G 'model:MX480' net.traceroute 8.8.8.8
    '''
    if proxy:
        return {'model': _get_grain(proxy, 'model')}


def serial(proxy):
    '''
    Returns the chassis serial number.

    Example: FOX1234W00F

    CLI Example - select all devices whose serial number begins with `FOX` and display the serial number value:

    .. code-block:: bash

        salt -G 'serial:FOX*' grains.get serial

    Output:

    .. code-block:: yaml

        edge01.icn01:
            FOXW00F001
        edge01.del01:
            FOXW00F002
        edge01.yyz01:
            FOXW00F003
        edge01.mrs01:
            FOXW00F004
    '''
    if proxy:
        return {'serial': _get_grain(proxy, 'serial_number')}


def vendor(proxy):
    '''
    Returns the network device vendor.

    Example: juniper, cisco, arista etc.

    CLI Example - select all devices produced by Cisco and shutdown:

    .. code-block:: bash

        salt -G 'vendor:cisco' net.cli "shut"
    '''
    if proxy:
        return {'vendor': _get_grain(proxy, 'vendor')}


def uptime(proxy):
    '''
    Returns the uptime in seconds.

    CLI Example - select all devices started/restarted within the last hour:

    .. code-block: bash

        salt -G 'uptime<3600' test.ping
    '''
    if proxy:
        return {'uptime': _get_grain(proxy, 'uptime')}


def interfaces(proxy):
    '''
    Returns the complete interfaces list of the network device.

    Example: ['lc-0/0/0', 'pfe-0/0/0', 'xe-1/3/0', 'lo0', 'irb', 'demux0', 'fxp0']

    CLI Example - select all devices that have a certain interface, e.g.: xe-1/1/1:

    .. code-block:: bash

        salt -G 'interfaces:xe-1/1/1' test.ping

    Output:

    .. code-block:: yaml

        edge01.yyz01:
            True
        edge01.maa01:
            True
        edge01.syd01:
            True
        edge01.del01:
            True
        edge01.dus01:
            True
        edge01.kix01:
            True
    '''
    if proxy:
        return {'interfaces': _get_grain(proxy, 'interface_list')}



# -*- coding: utf-8 -*-
'''
Module for interfacing to Junos devices.
'''

# Import python libraries
from __future__ import absolute_import
import logging
import json

# Import salt libraries
from salt.utils import fopen

try:
    from lxml import etree
except ImportError:
    from salt._compat import ElementTree as etree

# Juniper interface libraries
# https://github.com/Juniper/py-junos-eznc
try:
    # pylint: disable=W0611
    from jnpr.junos import Device
    from jnpr.junos.utils.sw import SW
    from jnpr.junos.utils.scp import SCP
    import jnpr.junos.utils
    import jnpr.junos.cfg
    import jxmlease
    # pylint: enable=W0611
    HAS_JUNOS = True
except ImportError:
    HAS_JUNOS = False

# Set up logging
log = logging.getLogger(__name__)


# Define the module's virtual name
__virtualname__ = 'junos'

__proxyenabled__ = ['junos']


def __virtual__():
    '''
    We need the Junos adapter libraries for this
    module to work.  We also need a proxymodule entry in __opts__
    in the opts dictionary
    '''
    if HAS_JUNOS and 'proxy' in __opts__:
        return __virtualname__
    else:
        return (False, 'The junos module could not be \
                loaded: junos-eznc or jxmlease or proxy could not be loaded.')


def facts_refresh():
    '''
    Reload the facts dictionary from the device.  Usually only needed
    if the device configuration is changed by some other actor.

    Usage:

    .. code-block:: bash

        salt 'device_name' junos.facts_refresh

    '''
    conn = __proxy__['junos.conn']()
    ret = dict()
    ret['out'] = True
    try:
        ret['message'] = conn.facts_refresh()

    except Exception as exception:
        ret['message'] = 'Execution failed due to "{0}"'.format(exception)
        ret['out'] = False

    return ret


def facts():
    '''
    Displays the facts gathered during the connection.

    Usage:

    .. code-block:: bash

        salt 'device_name' junos.facts

    '''
    conn = __proxy__['junos.conn']()
    ret = dict()
    ret['message'] = json.dumps(conn.facts)
    ret['out'] = True
    return ret


def rpc(cmd=None, dest=None, format='xml', *args, **kwargs):
    '''
    This function executes the rpc provided as arguments on the junos device.
    The returned data can be stored in a file whose destination can be
    specified with 'dest' keyword in the arguments.

    Usage:

    .. code-block:: bash

        salt 'device' junos.rpc 'get_config' 'text' filter='<configuration><system/></configuration>'

        salt 'device' junos.rpc 'get-interface-information' '/home/user/interface.log' interface_name='lo0' terse=True


    Options:
      * cmd: the rpc to be executed
      * dest: destination file where the rpc ouput is dumped
      * format: the format in which the rpc reply must be stored in file specified in the dest (used only when dest is specified)
      * args: other arguments as taken by rpc call of PyEZ
      * kwargs: keyworded arguments taken by rpc call of PyEZ
    '''

    conn = __proxy__['junos.conn']()
    ret = dict()
    ret['out'] = True

    op = dict()
    if '__pub_arg' in kwargs:
        if isinstance(kwargs['__pub_arg'][-1], dict):
            op.update(kwargs['__pub_arg'][-1])
    else:
        op.update(kwargs)

    if dest is None and format != 'xml':
        log.warning(
            'Format ignored as it is only used for output which is dumped in the file.')

    write_response = ''
    try:
        if cmd in ['get-config', 'get_config']:
            filter_reply = None
            if 'filter' in op:
                filter_reply = etree.XML(op['filter'])

            xml_reply = getattr(
                conn.rpc,
                cmd.replace('-',
                            '_'))(filter_reply,
                                  options=op)
            ret['message'] = jxmlease.parse(etree.tostring(xml_reply))
            write_response = etree.tostring(xml_reply)

            if dest is not None and format != 'xml':
                op.update({'format': format})
                rpc_reply = getattr(
                    conn.rpc,
                    cmd.replace('-',
                                '_'))(filter_reply,
                                      options=op)
                if format == 'json':
                    write_response = json.dumps(rpc_reply, indent=1)
                else:
                    write_response = rpc_reply.text
        else:

            xml_reply = getattr(conn.rpc, cmd.replace('-', '_'))(**op)
            ret['message'] = jxmlease.parse(etree.tostring(xml_reply))
            write_response = etree.tostring(xml_reply)

            if dest is not None and format != 'xml':
                rpc_reply = getattr(
                    conn.rpc,
                    cmd.replace('-',
                                '_'))({'format': format},
                                      **op)
                if format == 'json':
                    write_response = json.dumps(rpc_reply, indent=1)
                else:
                    write_response = rpc_reply.text

    except Exception as exception:
        ret['message'] = 'Execution failed due to "{0}"'.format(exception)
        ret['out'] = False

    if dest is not None:
        with fopen(dest, 'w') as fp:
            fp.write(write_response)

    return ret


def set_hostname(hostname=None, commit_change=True):
    '''
    To set the name of the device.

    Usage:

    .. code-block:: bash

        salt 'device_name' junos.set_hostname hostname=salt-device


    Options:
      * hostname: The name to be set.
      * commit_change: Whether to commit the changes.(default=True)
    '''
    conn = __proxy__['junos.conn']()
    ret = dict()
    if hostname is None:
        ret['out'] = False
        return ret

    # Added to recent versions of JunOs
    # Use text format instead
    set_string = 'set system host-name {0}'.format(hostname)
    conn.cu.load(set_string, format='set')
    if commit_change:
        return commit()
    else:
        ret['out'] = True
        ret['msg'] = 'set system host-name {0} is queued'.format(hostname)

    return ret


def commit():
    '''
    To commit the changes loaded in the candidate configuration.

    Usage:

    .. code-block:: bash

        salt 'device_name' junos.commit

    '''

    conn = __proxy__['junos.conn']()
    ret = {}
    commit_ok = conn.cu.commit_check()
    if commit_ok:
        try:
            conn.cu.commit(confirm=False)
            ret['out'] = True
            ret['message'] = 'Commit Successful.'
        except Exception as exception:
            ret['out'] = False
            ret['message'] = 'Pre-commit check succeeded but actual commit failed with "{0}"'.format(
                exception)
    else:
        ret['out'] = False
        ret['message'] = 'Pre-commit check failed.'

    return ret


def rollback():
    '''
    To rollback the last committed configuration changes

    Usage:

    .. code-block:: bash

        salt 'device_name' junos.rollback

    '''
    ret = dict()
    conn = __proxy__['junos.conn']()

    ret['out'] = conn.cu.rollback(0)

    if ret['out']:
        ret['message'] = 'Rollback successful'
    else:
        ret['message'] = 'Rollback failed'

    return ret


def diff():
    '''
    Gives the difference between the candidate and the current configuration.

    Usage:

    .. code-block:: bash

        salt 'device_name' junos.diff

    '''
    conn = __proxy__['junos.conn']()
    ret = dict()
    ret['out'] = True
    ret['message'] = conn.cu.diff()

    return ret


def ping():
    '''
    To check the connection with the device

    Usage:

    .. code-block:: bash

        salt 'device_name' junos.ping

    '''
    conn = __proxy__['junos.conn']()
    ret = dict()
    ret['message'] = conn.probe()
    if ret['message']:
        ret['out'] = True
    else:
        ret['out'] = False
    return ret


def cli(command=None):
    '''
    Executes the CLI commands and reuturns the text output.

    Usage:

    .. code-block:: bash

        salt 'device_name' junos.cli 'show version'


    Options:
      * command: The command that need to be executed on Junos CLI.
    '''
    conn = __proxy__['junos.conn']()
    ret = dict()
    ret['message'] = conn.cli(command)
    ret['out'] = True
    return ret


def shutdown(time=0):
    '''
    Shuts down the device after the given time.

    Usage:

    .. code-block:: bash

        salt 'device_name' junos.shutdown 10


    Options:
      * time: Time in seconds after which the device should shutdown (default=0)
    '''
    conn = __proxy__['junos.conn']()
    ret = dict()
    sw = SW(conn)
    try:
        shut = sw.poweroff()
        shut(time)
        ret['message'] = 'Successfully powered off.'
        ret['out'] = False
    except Exception as exception:
        ret['message'] = 'Could not poweroff'
        ret['out'] = False

    return ret


def install_config(path=None, **kwargs):
    '''
    Installs the given configuration file into the candidate configuration.
    Commits the changes if the commit checks or throws an error.

    Usage:

    .. code-block:: bash

        salt 'device_name' junos.install_config '/home/user/config.set' timeout=300


    Options:
      * path: Path where the configuration file is present.
      * kwargs: keyworded arguments taken by load fucntion of PyEZ
    '''
    conn = __proxy__['junos.conn']()
    ret = dict()
    ret['out'] = True

    if 'timeout' in kwargs:
        conn.timeout = kwargs['timeout']

    options = {'path': path}

    try:
        conn.cu.load(**options)
        conn.cu.pdiff()

    except Exception as exception:
        ret['message'] = 'Could not load configuration due to : "{0}"'.format(
            exception)
        ret['out'] = False

    if conn.cu.commit_check():
        ret['message'] = 'Successfully loaded and committed!'
        conn.cu.commit()
    else:
        ret['message'] = 'Commit check failed.'
        ret['out'] = False
        conn.cu.rollback()

    return ret


def zeroize():
    '''
    Resets the device to default factory settings

    Usage:

    .. code-block:: bash

        salt 'device_name' junos.zeroize

    '''
    conn = __proxy__['junos.conn']()
    ret = dict()
    ret['out'] = True
    try:
        conn.cli('request system zeroize')
        ret['message'] = 'Completed zeroize and rebooted'
    except Exception as exception:
        ret['message'] = 'Could not zeroize due to : "{0}"'.format(exception)
        ret['out'] = False

    return ret


def install_os(path=None, **kwargs):
    '''
    Installs the given image on the device. After the installation is complete the device is rebooted,
    if reboot=True is given as a keyworded argument.

    Usage:

    .. code-block:: bash

        salt 'device_name' junos.install_os '/home/user/junos_image.tgz' reboot=True


    Options
      * path: Path where the image file is present.
      * kwargs: keyworded arguments to be given such as timeout, reboot etc

    '''
    conn = __proxy__['junos.conn']()
    ret = dict()
    ret['out'] = True

    if 'timeout' in kwargs:
        conn.timeout = kwargs['timeout']

    try:
        install = conn.sw.install(path, progress=True)
        ret['message'] = 'Installed the os.'
    except Exception as exception:
        ret['message'] = 'Installation failed due to : "{0}"'.format(exception)
        ret['out'] = False

    if 'reboot' in kwargs and kwargs['reboot'] is True:
        rbt = conn.sw.reboot()
        ret['message'] = 'Successfully installed and rebooted!'

    return ret


def file_copy(src=None, dest=None):
    '''
    Copies the file from the local device to the junos device.

    Usage:

    .. code-block:: bash

        salt 'device_name' junos.file_copy /home/m2/info.txt info_copy.txt


    Options
      * src: The sorce path where the file is kept.
      * dest: The destination path where the file will be copied.
    '''
    conn = __proxy__['junos.conn']()
    ret = dict()
    ret['out'] = True
    try:
        with SCP(conn, progress=True) as scp:
            scp.put(src, dest)
        ret['message'] = 'Successfully copied file from {0} to {1}'.format(
            src, dest)

    except Exception as exception:
        ret['message'] = 'Could not copy file : "{0}"'.format(exception)
        ret['out'] = False

    return ret



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import pecl

pecl.__salt__ = {}
pecl.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PeclTestCase(TestCase):
    '''
    Test cases for salt.states.pecl
    '''
    # 'installed' function tests: 1

    def test_installed(self):
        '''
        Test to make sure that a pecl extension is installed.
        '''
        name = 'mongo'
        ver = '1.0.1'

        ret = {'name': name,
               'result': None,
               'comment': '',
               'changes': {}}

        mock_lst = MagicMock(return_value={name: 'stable'})
        mock_t = MagicMock(return_value=True)
        with patch.dict(pecl.__salt__, {'pecl.list': mock_lst,
                                        'pecl.install': mock_t}):
            comt = ('Pecl extension {0} is already installed.'.format(name))
            ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(pecl.installed(name), ret)

            with patch.dict(pecl.__opts__, {'test': True}):
                comt = ('Pecl extension mongo-1.0.1 would have been installed')
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(pecl.installed(name, version=ver), ret)

            with patch.dict(pecl.__opts__, {'test': False}):
                comt = ('Pecl extension mongo-1.0.1 was successfully installed')
                ret.update({'comment': comt, 'result': True,
                            'changes': {'mongo-1.0.1': 'Installed'}})
                self.assertDictEqual(pecl.installed(name, version=ver), ret)

    # 'removed' function tests: 1

    def test_removed(self):
        '''
        Test to make sure that a pecl extension is not installed.
        '''
        name = 'mongo'

        ret = {'name': name,
               'result': None,
               'comment': '',
               'changes': {}}

        mock_lst = MagicMock(side_effect=[{}, {name: 'stable'},
                                          {name: 'stable'}])
        mock_t = MagicMock(return_value=True)
        with patch.dict(pecl.__salt__, {'pecl.list': mock_lst,
                                        'pecl.uninstall': mock_t}):
            comt = ('Pecl extension {0} is not installed.'.format(name))
            ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(pecl.removed(name), ret)

            with patch.dict(pecl.__opts__, {'test': True}):
                comt = ('Pecl extension mongo would have been removed')
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(pecl.removed(name), ret)

            with patch.dict(pecl.__opts__, {'test': False}):
                comt = ('Pecl extension mongo was successfully removed.')
                ret.update({'comment': comt, 'result': True,
                            'changes': {name: 'Removed'}})
                self.assertDictEqual(pecl.removed(name), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(PeclTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import smtp

smtp.__salt__ = {}


class SMTPRecipientsRefused(Exception):
    '''
    Mock SMTPRecipientsRefused class
    '''
    def __init__(self, msg):
        super(SMTPRecipientsRefused, self).__init__(msg)
        self.smtp_error = msg


class SMTPHeloError(Exception):
    '''
    Mock SMTPHeloError class
    '''
    def __init__(self, msg):
        super(SMTPHeloError, self).__init__(msg)
        self.smtp_error = msg


class SMTPSenderRefused(Exception):
    '''
    Mock SMTPSenderRefused class
    '''
    def __init__(self, msg):
        super(SMTPSenderRefused, self).__init__(msg)
        self.smtp_error = msg


class SMTPDataError(Exception):
    '''
    Mock SMTPDataError class
    '''
    def __init__(self, msg):
        super(SMTPDataError, self).__init__(msg)
        self.smtp_error = msg


class SMTPException(Exception):
    '''
    Mock SMTPException class
    '''
    def __init__(self, msg):
        super(SMTPException, self).__init__(msg)
        self.smtp_error = msg


class SMTPAuthenticationError(Exception):
    '''
    Mock SMTPAuthenticationError class
    '''
    def __init__(self, msg):
        super(SMTPAuthenticationError, self).__init__(msg)
        self.smtp_error = msg


class MockSMTPSSL(object):
    '''
    Mock SMTP_SSL class
    '''
    flag = None

    def __init__(self, server):
        pass

    def sendmail(self, sender, recipient, msg):
        '''
        Mock sendmail method
        '''
        if self.flag == 1:
            raise SMTPRecipientsRefused('All recipients were refused.')
        elif self.flag == 2:
            raise SMTPHeloError('Helo error')
        elif self.flag == 3:
            raise SMTPSenderRefused('Sender Refused')
        elif self.flag == 4:
            raise SMTPDataError('Data error')
        return (sender, recipient, msg)

    def login(self, username, password):
        '''
        Mock login method
        '''
        if self.flag == 5:
            raise SMTPAuthenticationError('SMTP Authentication Failure')
        return (username, password)

    @staticmethod
    def quit():
        '''
        Mock quit method
        '''
        return True


class MockSMTP(object):
    '''
    Mock SMTP class
    '''
    flag = None

    def __init__(self, server):
        pass

    @staticmethod
    def ehlo():
        '''
        Mock ehlo method
        '''
        return True

    @staticmethod
    def has_extn(name):
        '''
        Mock has_extn method
        '''
        return name

    def starttls(self):
        '''
        Mock starttls method
        '''
        if self.flag == 1:
            raise SMTPHeloError('Helo error')
        elif self.flag == 2:
            raise SMTPException('Exception error')
        elif self.flag == 3:
            raise RuntimeError
        return True

    def sendmail(self, sender, recipient, msg):
        '''
        Mock sendmail method
        '''
        if self.flag == 1:
            raise SMTPRecipientsRefused('All recipients were refused.')
        elif self.flag == 2:
            raise SMTPHeloError('Helo error')
        elif self.flag == 3:
            raise SMTPSenderRefused('Sender Refused')
        elif self.flag == 4:
            raise SMTPDataError('Data error')
        return (sender, recipient, msg)

    @staticmethod
    def quit():
        '''
        Mock quit method
        '''
        return True


class MockGaierror(Exception):
    '''
    Mock MockGaierror class
    '''
    def __init__(self, msg):
        super(MockGaierror, self).__init__(msg)
        self.smtp_error = msg


class MockSocket(object):
    '''
    Mock Socket class
    '''
    def __init__(self):
        self.gaierror = MockGaierror


class MockSmtplib(object):
    '''
    Mock smtplib class
    '''
    flag = None

    def __init__(self):
        self.SMTPRecipientsRefused = SMTPRecipientsRefused
        self.SMTPHeloError = SMTPHeloError
        self.SMTPSenderRefused = SMTPSenderRefused
        self.SMTPDataError = SMTPDataError
        self.SMTPException = SMTPException
        self.SMTPAuthenticationError = SMTPAuthenticationError
        self.server = None

    def SMTP_SSL(self, server):
        '''
        Mock SMTP_SSL method
        '''
        self.server = server
        if self.flag == 1:
            raise MockGaierror('gaierror')
        return MockSMTPSSL('server')

    def SMTP(self, server):
        '''
        Mock SMTP method
        '''
        self.server = server
        if self.flag == 1:
            raise MockGaierror('gaierror')
        return MockSMTP('server')

smtp.smtplib = MockSmtplib()
smtp.socket = MockSocket()


@skipIf(NO_MOCK, NO_MOCK_REASON)
class SmtpTestCase(TestCase):
    '''
    TestCase for salt.modules.smtp
    '''
    # 'send_msg' function tests: 1

    def test_send_msg(self):
        '''
        Tests if it send a message to an SMTP recipient.
        '''
        mock = MagicMock(return_value={'smtp.server': '', 'smtp.tls': 'True',
                                       'smtp.sender': '', 'smtp.username': '',
                                       'smtp.password': ''})
        with patch.dict(smtp.__salt__, {'config.option': mock}):
            self.assertTrue(smtp.send_msg('admin@example.com',
                                          'This is a salt module test',
                                          profile='my-smtp-account'))

            MockSMTPSSL.flag = 1
            self.assertFalse(smtp.send_msg('admin@example.com',
                                           'This is a salt module test',
                                           profile='my-smtp-account'))

            MockSMTPSSL.flag = 2
            self.assertFalse(smtp.send_msg('admin@example.com',
                                           'This is a salt module test',
                                           profile='my-smtp-account'))

            MockSMTPSSL.flag = 3
            self.assertFalse(smtp.send_msg('admin@example.com',
                                           'This is a salt module test',
                                           profile='my-smtp-account'))

            MockSMTPSSL.flag = 4
            self.assertFalse(smtp.send_msg('admin@example.com',
                                           'This is a salt module test',
                                           profile='my-smtp-account'))

        mock = MagicMock(return_value={'smtp.server': '', 'smtp.tls': '',
                                       'smtp.sender': '', 'smtp.username': '',
                                       'smtp.password': ''})
        with patch.dict(smtp.__salt__, {'config.option': mock}):
            MockSMTPSSL.flag = 5
            self.assertFalse(smtp.send_msg('admin@example.com',
                                           'This is a salt module test',
                                           username='myuser',
                                           password='verybadpass',
                                           sender='admin@example.com',
                                           server='smtp.domain.com'))

            MockSMTP.flag = 1
            self.assertFalse(smtp.send_msg('admin@example.com',
                                           'This is a salt module test',
                                           profile='my-smtp-account'))

            MockSMTP.flag = 2
            self.assertFalse(smtp.send_msg('admin@example.com',
                                           'This is a salt module test',
                                           profile='my-smtp-account'))

            MockSMTP.flag = 3
            self.assertFalse(smtp.send_msg('admin@example.com',
                                           'This is a salt module test',
                                           profile='my-smtp-account'))

            MockSmtplib.flag = 1
            self.assertFalse(smtp.send_msg('admin@example.com',
                                           'This is a salt module test',
                                           profile='my-smtp-account'))


if __name__ == '__main__':
    from integration import run_tests
    run_tests(SmtpTestCase, needs_daemon=False)



'''
Short-URL redirects
'''
import json
import os

import sphinx.ext.intersphinx

DOCS_URL = 'http://docs.saltstack.com/en/latest/'

def write_urls_index(app, exc):
    '''
    Generate a JSON file to serve as an index for short-URL lookups
    '''
    inventory = os.path.join(app.builder.outdir, 'objects.inv')
    objects = sphinx.ext.intersphinx.fetch_inventory(app, DOCS_URL, inventory)

    with open(os.path.join(app.builder.outdir, 'shorturls.json'), 'w') as f:
        json.dump(objects, f)

def setup(app):
    app.connect('build-finished', write_urls_index)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Thayne Harbaugh (tharbaug@adobe.com)`

    tests.integration.shell.proxy
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import python libs
from __future__ import absolute_import
import logging

# Import Salt Testing libs
from salttesting.helpers import ensure_in_syspath
ensure_in_syspath('../../')

# Import salt libs
import integration
import integration.utils
from integration.utils import testprogram

log = logging.getLogger(__name__)


class ProxyTest(testprogram.TestProgramCase):
    '''
    Various integration tests for the salt-proxy executable.
    '''

    def test_exit_status_no_proxyid(self):
        '''
        Ensure correct exit status when --proxyid argument is missing.
        '''

        proxy = testprogram.TestDaemonSaltProxy(
            name='proxy-no_proxyid',
            parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        proxy.setup()
        stdout, stderr, status = proxy.run(
            args=[
                '--config-dir', proxy.abs_path(proxy.config_dir),  # Needed due to verbatim_args=True
                '-d',
            ],
            verbatim_args=True,   # prevents --proxyid from being added automatically
            catch_stderr=True,
            with_retcode=True,
            # The proxy minion had a bug where it would loop forever
            # without daemonizing - protect that with a timeout.
            timeout=60,
        )
        self.assert_exit_status(
            status, 'EX_USAGE',
            message='no --proxyid specified',
            stdout=stdout,
            stderr=integration.utils.decode_byte_list(stderr)
        )
        # proxy.shutdown() should be unnecessary since the start-up should fail

    def test_exit_status_unknown_user(self):
        '''
        Ensure correct exit status when the proxy is configured to run as an unknown user.
        '''

        proxy = testprogram.TestDaemonSaltProxy(
            name='proxy-unknown_user',
            config_base={'user': 'some_unknown_user_xyz'},
            parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        proxy.setup()
        stdout, stderr, status = proxy.run(
            args=['-d'],
            catch_stderr=True,
            with_retcode=True,
        )
        self.assert_exit_status(
            status, 'EX_NOUSER',
            message='unknown user not on system',
            stdout=stdout,
            stderr=integration.utils.decode_byte_list(stderr)
        )
        # proxy.shutdown() should be unnecessary since the start-up should fail

    # pylint: disable=invalid-name
    def test_exit_status_unknown_argument(self):
        '''
        Ensure correct exit status when an unknown argument is passed to salt-proxy.
        '''

        proxy = testprogram.TestDaemonSaltProxy(
            name='proxy-unknown_argument',
            parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        proxy.setup()
        stdout, stderr, status = proxy.run(
            args=['-d', '--unknown-argument'],
            catch_stderr=True,
            with_retcode=True,
        )
        self.assert_exit_status(
            status, 'EX_USAGE',
            message='unknown argument',
            stdout=stdout, stderr=stderr
        )
        # proxy.shutdown() should be unnecessary since the start-up should fail

    def test_exit_status_correct_usage(self):
        '''
        Ensure correct exit status when salt-proxy starts correctly.
        '''

        proxy = testprogram.TestDaemonSaltProxy(
            name='proxy-correct_usage',
            parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        proxy.setup()
        stdout, stderr, status = proxy.run(
            args=['-d'],
            catch_stderr=True,
            with_retcode=True,
        )
        self.assert_exit_status(
            status, 'EX_OK',
            message='correct usage',
            stdout=stdout,
            stderr=integration.utils.decode_byte_list(stderr)
        )
        proxy.shutdown()


if __name__ == '__main__':
    integration.run_tests(ProxyTest)



# -*- coding: utf-8 -*-
'''
Integration tests for timezone module

Linux and Solaris are supported
'''

# Import python libs
from __future__ import absolute_import

# Import Salt Testing libs
from salttesting.helpers import ensure_in_syspath
ensure_in_syspath('../../')

# Import salt libs
import integration


class TimezoneLinuxModuleTest(integration.ModuleCase):
    def setUp(self):
        '''
        Set up Linux test environment
        '''
        ret_grain = self.run_function('grains.item', ['kernel'])
        if 'Linux' not in ret_grain['kernel']:
            self.skipTest('For Linux only')
        super(TimezoneLinuxModuleTest, self).setUp()

    def test_get_hwclock(self):
        timescale = ['UTC', 'localtime']
        ret = self.run_function('timezone.get_hwclock')
        self.assertIn(ret, timescale)


class TimezoneSolarisModuleTest(integration.ModuleCase):
    def setUp(self):
        '''
        Set up Solaris test environment
        '''
        ret_grain = self.run_function('grains.item', ['os_family'])
        if 'Solaris' not in ret_grain['os_family']:
            self.skipTest('For Solaris only')
        super(TimezoneSolarisModuleTest, self).setUp()

    def test_get_hwclock(self):
        timescale = ['UTC', 'localtime']
        ret = self.run_function('timezone.get_hwclock')
        self.assertIn(ret, timescale)


if __name__ == '__main__':
    from integration import run_tests
    run_tests([TimezoneLinuxModuleTest, TimezoneSolarisModuleTest])



# -*- coding: utf-8 -*-
'''
    :synopsis: Unit Tests for Windows Registry Module 'module.reg'
    :platform: Windows
    :maturity: develop
    :codeauthor: Damon Atkins <https://github.com/damon-atkins>
    versionadded:: 2016.11.0
'''
# Import Python future libs
from __future__ import absolute_import
from __future__ import unicode_literals
# Import Python Libs
import sys
import time
# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.helpers import destructiveTest
# Import Salt Libs
from salt.modules import reg as win_mod_reg
from salt.ext import six
try:
    from salt.ext.six.moves import winreg as _winreg  # pylint: disable=import-error,no-name-in-module
    NO_WINDOWS_MODULES = False
except ImportError:
    NO_WINDOWS_MODULES = True

PY2 = sys.version_info[0] == 2
# The following used to make sure we are not
# testing already existing data
# Note strftime retunrns a str, so we need to make it unicode
TIMEINT = int(time.time())

if PY2:
    TIME_INT_UNICODE = six.text_type(TIMEINT)
    TIMESTR = time.strftime('%X %x %Z').decode('utf-8')
else:
    TIMESTR = time.strftime('%X %x %Z')
    TIME_INT_UNICODE = str(TIMEINT)  # pylint: disable=R0204


# we do not need to prefix this with u, as we are
# using from __future__ import unicode_literals
UNICODETEST_WITH_SIGNS = 'Testing Unicode \N{COPYRIGHT SIGN},\N{TRADE MARK SIGN},\N{REGISTERED SIGN} '+TIMESTR
UNICODETEST_WITHOUT_SIGNS = 'Testing Unicode'+TIMESTR
UNICODE_TEST_KEY = 'UnicodeKey \N{TRADE MARK SIGN} '+TIME_INT_UNICODE
UNICODE_TEST_KEY_DEL = 'Delete Me \N{TRADE MARK SIGN} '+TIME_INT_UNICODE


@skipIf(NO_WINDOWS_MODULES, 'requires Windows OS to test Windows registry')
class RegWinTestCase(TestCase):
    '''
    Test cases for salt.modules.reg
    '''

    @skipIf(not sys.platform.startswith("win"), "requires Windows OS")
    def test_read_reg_plain(self):
        '''
        Test - Read a registry value from a subkey using Pythen 2 Strings or
        Pythen 3 Bytes
        '''
        if not PY2:
            self.skipTest('Invalid for Python Version 2')

        subkey = b'Software\\Microsoft\\Windows NT\\CurrentVersion'
        vname = b'PathName'
        handle = _winreg.OpenKey(
                    _winreg.HKEY_LOCAL_MACHINE,
                    subkey,
                    0,
                    _winreg.KEY_ALL_ACCESS
                    )
        (current_vdata, dummy_current_vtype) = _winreg.QueryValueEx(handle, vname)
        _winreg.CloseKey(handle)

        test_vdata = win_mod_reg.read_value(b'HKEY_LOCAL_MACHINE', subkey, vname)[b'vdata']
        self.assertEqual(
            test_vdata, current_vdata)

    @skipIf(not sys.platform.startswith("win"), "requires Windows OS")
    def test_read_reg_unicode(self):
        '''
        Test - Read a registry value from a subkey using Pythen 2 Unicode
        or Pythen 3 Str i.e. Unicode
        '''
        subkey = 'Software\\Microsoft\\Windows NT\\CurrentVersion'
        vname = 'PathName'
        handle = _winreg.OpenKey(
                    _winreg.HKEY_LOCAL_MACHINE,
                    subkey,
                    0,
                    _winreg.KEY_ALL_ACCESS
                    )
        (current_vdata, dummy_current_vtype) = _winreg.QueryValueEx(handle, vname)
        _winreg.CloseKey(handle)

        test_vdata = win_mod_reg.read_value(
                        'HKEY_LOCAL_MACHINE',
                        subkey,
                        vname)['vdata']
        self.assertEqual(test_vdata, current_vdata)

    @skipIf(not sys.platform.startswith("win"), "requires Windows OS")
    def test_list_keys_fail(self):
        '''
        Test - Read list the keys under a subkey which does not exist.
        '''
        subkey = 'ThisIsJunkItDoesNotExistIhope'
        test_list = win_mod_reg.list_keys('HKEY_LOCAL_MACHINE', subkey)
        # returns a tuple with first item false, and second item a reason
        test = isinstance(test_list, tuple) and (not test_list[0])
        self.assertTrue(test)

    @skipIf(not sys.platform.startswith("win"), "requires Windows OS")
    def test_list_keys(self):
        '''
        Test - Read list the keys under a subkey
        '''
        subkey = 'Software\\Microsoft\\Windows NT\\CurrentVersion'
        test_list = win_mod_reg.list_keys('HKEY_LOCAL_MACHINE', subkey)
        test = len(test_list) > 5  # Their should be a lot more than 5 items
        self.assertTrue(test)

    @skipIf(not sys.platform.startswith("win"), "requires Windows OS")
    def test_list_values_fail(self):
        '''
        Test - List the values under a subkey which does not exist.
        '''
        subkey = 'ThisIsJunkItDoesNotExistIhope'
        test_list = win_mod_reg.list_values('HKEY_LOCAL_MACHINE', subkey)
        # returns a tuple with first item false, and second item a reason
        test = isinstance(test_list, tuple) and (not test_list[0])
        self.assertTrue(test)

    @skipIf(not sys.platform.startswith("win"), "requires Windows OS")
    def test_list_values(self):
        '''
        Test - List the values under a subkey.
        '''
        subkey = r'Software\Microsoft\Windows NT\CurrentVersion'
        test_list = win_mod_reg.list_values('HKEY_LOCAL_MACHINE', subkey)
        test = len(test_list) > 5  # There should be a lot more than 5 items
        self.assertTrue(test)

    # Not considering this destructive as its writing to a private space
    @skipIf(not sys.platform.startswith("win"), "requires Windows OS")
    def test_set_value_unicode(self):
        '''
        Test - set a registry plain text subkey name to a unicode string value
        '''
        vname = 'TestUniccodeString'
        subkey = 'Software\\SaltStackTest'
        test1_success = False
        test2_success = False
        test1_success = win_mod_reg.set_value(
                            'HKEY_LOCAL_MACHINE',
                            subkey,
                            vname,
                            UNICODETEST_WITH_SIGNS
                            )
        # Now use _winreg direct to see if it worked as expected
        if test1_success:
            handle = _winreg.OpenKey(
                        _winreg.HKEY_LOCAL_MACHINE,
                        subkey,
                        0,
                        _winreg.KEY_ALL_ACCESS
                        )
            (current_vdata, dummy_current_vtype) = _winreg.QueryValueEx(handle, vname)
            _winreg.CloseKey(handle)
        test2_success = (current_vdata == UNICODETEST_WITH_SIGNS)
        self.assertTrue(test1_success and test2_success)

    @skipIf(not sys.platform.startswith("win"), "requires Windows OS")
    def test_set_value_unicode_key(self):
        '''
        Test - set a registry Unicode subkey name with unicode characters within
        to a integer
        '''
        test_success = win_mod_reg.set_value(
                        'HKEY_LOCAL_MACHINE',
                        'Software\\SaltStackTest',
                        UNICODE_TEST_KEY,
                        TIMEINT,
                        'REG_DWORD'
                        )
        self.assertTrue(test_success)

    @skipIf(not sys.platform.startswith("win"), "requires Windows OS")
    def test_del_value(self):
        '''
        Test - Create Directly and Delete with salt a registry value
        '''
        subkey = 'Software\\SaltStackTest'
        vname = UNICODE_TEST_KEY_DEL
        vdata = 'I will be deleted'
        if PY2:
            handle = _winreg.CreateKeyEx(
                        _winreg.HKEY_LOCAL_MACHINE,
                        subkey.encode('mbcs'),
                        0,
                        _winreg.KEY_ALL_ACCESS
                        )
            _winreg.SetValueEx(
                handle,
                vname.encode('mbcs'),
                0,
                _winreg.REG_SZ,
                vdata.encode('mbcs')
                )
        else:
            handle = _winreg.CreateKeyEx(
                        _winreg.HKEY_LOCAL_MACHINE,
                        subkey,
                        0,
                        _winreg.KEY_ALL_ACCESS
                        )
            _winreg.SetValueEx(handle, vname, 0, _winreg.REG_SZ, vdata)
        _winreg.CloseKey(handle)
        # time.sleep(15) # delays for 15 seconds
        test_success = win_mod_reg.delete_value(
                        'HKEY_LOCAL_MACHINE',
                        subkey,
                        vname
                        )
        self.assertTrue(test_success)

    @skipIf(not sys.platform.startswith("win"), "requires Windows OS")
    def test_del_key_recursive_user(self):
        '''
        Test - Create directly key/value pair and Delete recusivly with salt
        '''
        subkey = 'Software\\SaltStackTest'
        vname = UNICODE_TEST_KEY_DEL
        vdata = 'I will be deleted recursive'
        if PY2:
            handle = _winreg.CreateKeyEx(
                        _winreg.HKEY_CURRENT_USER,
                        subkey.encode('mbcs'),
                        0,
                        _winreg.KEY_ALL_ACCESS
                        )
            _winreg.SetValueEx(
                handle,
                vname.encode('mbcs'),
                0,
                _winreg.REG_SZ,
                vdata.encode('mbcs')
                )
        else:
            handle = _winreg.CreateKeyEx(
                        _winreg.HKEY_CURRENT_USER,
                        subkey,
                        0,
                        _winreg.KEY_ALL_ACCESS
                        )
            _winreg.SetValueEx(handle, vname, 0, _winreg.REG_SZ, vdata)
        _winreg.CloseKey(handle)
        # time.sleep(15) # delays for 15 seconds so you can run regedit & watch it happen
        test_success = win_mod_reg.delete_key_recursive('HKEY_CURRENT_USER', subkey)
        self.assertTrue(test_success)

    @skipIf(not sys.platform.startswith("win"), "requires Windows OS")
    @destructiveTest
    def test_del_key_recursive_machine(self):
        '''
        This is a DESTRUCTIVE TEST it creates a new registry entry.
        And then destroys the registry entry recusively , however it is completed in its own space
        within the registry. We mark this as destructiveTest as it has the potential
        to detroy a machine if salt reg code has a large error in it.
        '''
        subkey = 'Software\\SaltStackTest'
        vname = UNICODE_TEST_KEY_DEL
        vdata = 'I will be deleted recursive'
        if PY2:
            handle = _winreg.CreateKeyEx(
                        _winreg.HKEY_LOCAL_MACHINE,
                        subkey.encode('mbcs'),
                        0,
                        _winreg.KEY_ALL_ACCESS
                        )
            _winreg.SetValueEx(
                        handle,
                        vname.encode('mbcs'),
                        0,
                        _winreg.REG_SZ,
                        vdata.encode('mbcs')
                        )
        else:
            handle = _winreg.CreateKeyEx(
                        _winreg.HKEY_LOCAL_MACHINE,
                        subkey,
                        0,
                        _winreg.KEY_ALL_ACCESS
                        )
            _winreg.SetValueEx(handle, vname, 0, _winreg.REG_SZ, vdata)
        _winreg.CloseKey(handle)
        # time.sleep(15) # delays for 15 seconds so you can run regedit and watch it happen
        test_success = win_mod_reg.delete_key_recursive('HKEY_LOCAL_MACHINE', subkey)
        self.assertTrue(test_success)

    # pylint: disable=W0511
    # TODO: Test other hives, other than HKEY_LOCAL_MACHINE and HKEY_CURRENT_USER

if __name__ == '__main__':
    from integration import run_tests  # pylint: disable=C0413
    run_tests(RegWinTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-

from __future__ import absolute_import

# Import Salt Testing libs
from salttesting.helpers import (
    destructiveTest,
    requires_network,
    requires_salt_modules,
    ensure_in_syspath
)
ensure_in_syspath('../../')

# Import salt libs
import integration


class PkgModuleTest(integration.ModuleCase,
                    integration.SaltReturnAssertsMixIn):
    '''
    Validate the pkg module
    '''
    def test_list(self):
        '''
        verify that packages are installed
        '''
        ret = self.run_function('pkg.list_pkgs')
        self.assertNotEqual(len(ret.keys()), 0)

    def test_version_cmp(self):
        '''
        test package version comparison on supported platforms
        '''
        func = 'pkg.version_cmp'
        os_family = self.run_function('grains.item', ['os_family'])['os_family']
        if os_family == 'Debian':
            lt = ['0.2.4-0ubuntu1', '0.2.4.1-0ubuntu1']
            eq = ['0.2.4-0ubuntu1', '0.2.4-0ubuntu1']
            gt = ['0.2.4.1-0ubuntu1', '0.2.4-0ubuntu1']

            self.assertEqual(self.run_function(func, lt), -1)
            self.assertEqual(self.run_function(func, eq), 0)
            self.assertEqual(self.run_function(func, gt), 1)
        elif os_family == 'Suse':
            lt = ['2.3.0-1', '2.3.1-15.1']
            eq = ['2.3.1-15.1', '2.3.1-15.1']
            gt = ['2.3.2-15.1', '2.3.1-15.1']

            self.assertEqual(self.run_function(func, lt), -1)
            self.assertEqual(self.run_function(func, eq), 0)
            self.assertEqual(self.run_function(func, gt), 1)
        else:
            self.skipTest('{0} is unavailable on {1}'.format(func, os_family))

    @requires_network()
    @destructiveTest
    def test_mod_del_repo(self):
        '''
        test modifying and deleting a software repository
        '''
        os_grain = self.run_function('grains.item', ['os'])['os']

        try:
            repo = None
            if os_grain == 'Ubuntu':
                repo = 'ppa:otto-kesselgulasch/gimp-edge'
                uri = 'http://ppa.launchpad.net/otto-kesselgulasch/gimp-edge/ubuntu'
                ret = self.run_function('pkg.mod_repo', [repo, 'comps=main'])
                self.assertNotEqual(ret, {})
                ret = self.run_function('pkg.get_repo', [repo])
                self.assertEqual(ret['uri'], uri)
            elif os_grain == 'CentOS':
                major_release = int(
                    self.run_function(
                        'grains.item',
                        ['osmajorrelease']
                    )['osmajorrelease']
                )
                repo = 'saltstack'
                name = 'SaltStack repo for RHEL/CentOS {0}'.format(major_release)
                baseurl = 'http://repo.saltstack.com/yum/redhat/{0}/x86_64/latest/'.format(major_release)
                gpgkey = 'https://repo.saltstack.com/yum/rhel{0}/SALTSTACK-GPG-KEY.pub'.format(major_release)
                gpgcheck = 1
                enabled = 1
                ret = self.run_function(
                    'pkg.mod_repo',
                    [repo],
                    name=name,
                    baseurl=baseurl,
                    gpgkey=gpgkey,
                    gpgcheck=gpgcheck,
                    enabled=enabled,
                )
                # return data from pkg.mod_repo contains the file modified at
                # the top level, so use next(iter(ret)) to get that key
                self.assertNotEqual(ret, {})
                repo_info = ret[next(iter(ret))]
                self.assertIn(repo, repo_info)
                self.assertEqual(repo_info[repo]['baseurl'], baseurl)
                ret = self.run_function('pkg.get_repo', [repo])
                self.assertEqual(ret['baseurl'], baseurl)
        finally:
            if repo is not None:
                self.run_function('pkg.del_repo', [repo])

    def test_owner(self):
        '''
        test finding the package owning a file
        '''
        func = 'pkg.owner'
        available = self.run_function('sys.doc', [func])

        if available:
            ret = self.run_function(func, ['/bin/ls'])
            self.assertNotEqual(len(ret), 0)
        else:
            os_grain = self.run_function('grains.item', ['os'])['os']
            self.skipTest('{0} is unavailable on {1}'.format(func, os_grain))

    @requires_network()
    @destructiveTest
    def test_install_remove(self):
        '''
        successfully install and uninstall a package
        '''
        pkg = 'htop'
        version = self.run_function('pkg.version', [pkg])
        os_grain = self.run_function('grains.item', ['os'])['os']
        os_release = self.run_function('grains.item', ['osrelease'])['osrelease']

        if os_grain == 'Ubuntu':
            if os_release.startswith('12.'):
                self.skipTest('pkg.install and pkg.remove do not work on ubuntu '
                              '12 when run from the test suite')

        def test_install():
            install_ret = self.run_function('pkg.install', [pkg])
            self.assertIn(pkg, install_ret)

        def test_remove():
            remove_ret = self.run_function('pkg.remove', [pkg])
            self.assertIn(pkg, remove_ret)

        if version:
            test_remove()
            test_install()
        else:
            test_install()
            test_remove()

    @requires_salt_modules('pkg.hold')
    @requires_network()
    @destructiveTest
    def test_hold_unhold(self):
        '''
        test holding and unholding a package
        '''
        pkg = 'htop'
        os_family = self.run_function('grains.item', ['os_family'])['os_family']
        os_major_release = self.run_function('grains.item', ['osmajorrelease'])['osmajorrelease']
        available = self.run_function('sys.doc', ['pkg.hold'])

        if os_family == 'RedHat':
            if os_major_release == '5':
                self.skipTest('`yum versionlock` does not seem to work on RHEL/CentOS 5')

        if available:
            if os_family == 'RedHat':
                lock_pkg = 'yum-versionlock' if os_major_release == '5' else 'yum-plugin-versionlock'
                versionlock = self.run_function('pkg.version', [lock_pkg])
                if not versionlock:
                    self.run_function('pkg.install', [lock_pkg])

            hold_ret = self.run_function('pkg.hold', [pkg])
            self.assertIn(pkg, hold_ret)
            self.assertTrue(hold_ret[pkg]['result'])

            unhold_ret = self.run_function('pkg.unhold', [pkg])
            self.assertIn(pkg, unhold_ret)
            self.assertTrue(hold_ret[pkg]['result'])

            if os_family == 'RedHat':
                if not versionlock:
                    self.run_function('pkg.remove', [lock_pkg])

        else:
            os_grain = self.run_function('grains.item', ['os'])['os']
            self.skipTest('{0} is unavailable on {1}'.format('pkg.hold', os_grain))

    @requires_network()
    @destructiveTest
    def test_refresh_db(self):
        '''
        test refreshing the package database
        '''
        func = 'pkg.refresh_db'
        os_family = self.run_function('grains.item', ['os_family'])['os_family']

        if os_family == 'RedHat':
            ret = self.run_function(func)
            self.assertIn(ret, (True, None))
        elif os_family == 'Suse':
            ret = self.run_function(func)
            if not isinstance(ret, dict):
                self.skipTest('Upstream repo did not return coherent results. Skipping test.')
            self.assertNotEqual(ret, {})
        elif os_family == 'Debian':
            ret = self.run_function(func)
            if not isinstance(ret, dict):
                self.skipTest('{0} encountered an error: {1}'.format(func, ret))
            self.assertNotEqual(ret, {})
            if not isinstance(ret, dict):
                self.skipTest('Upstream repo did not return coherent results. Skipping test.')
            for source, state in ret.items():
                self.assertIn(state, (True, False, None))
        else:
            os_grain = self.run_function('grains.item', ['os'])['os']
            self.skipTest('{0} is unavailable on {1}'.format(func, os_grain))

    @requires_salt_modules('pkg.info_installed')
    def test_pkg_info(self):
        '''
        Test returning useful information on Ubuntu systems.
        '''
        func = 'pkg.info_installed'
        os_family = self.run_function('grains.item', ['os_family'])['os_family']

        if os_family == 'Debian':
            ret = self.run_function(func, ['bash-completion', 'dpkg'])
            keys = ret.keys()
            self.assertIn('bash-completion', keys)
            self.assertIn('dpkg', keys)
        elif os_family == 'RedHat':
            ret = self.run_function(func, ['rpm', 'yum'])
            keys = ret.keys()
            self.assertIn('rpm', keys)
            self.assertIn('yum', keys)
        elif os_family == 'SUSE':
            ret = self.run_function(func, ['less', 'zypper'])
            keys = ret.keys()
            self.assertIn('less', keys)
            self.assertIn('zypper', keys)

    @requires_network()
    @destructiveTest
    def test_pkg_upgrade_has_pending_upgrades(self):
        '''
        Test running a system upgrade when there are packages that need upgrading
        '''
        func = 'pkg.upgrade'
        os_family = self.run_function('grains.item', ['os_family'])['os_family']

        # First make sure that an up-to-date copy of the package db is available
        self.run_function('pkg.refresh_db')

        if os_family == 'Suse':
            # pkg.latest version returns empty if the latest version is already installed
            vim_version_dict = self.run_function('pkg.latest_version', ['vim'])
            vim_info = self.run_function('pkg.info_available', ['vim'])['vim']
            if vim_version_dict == {}:
                # Latest version is installed, get its version and construct
                # a version selector so the immediately previous version is selected
                vim_version = 'version=<'+vim_info['version']
            else:
                # Vim was not installed, so pkg.latest_version returns the latest one.
                # Construct a version selector so immediately previous version is selected
                vim_version = 'version=<'+vim_version_dict

            # Only install a new version of vim if vim is up-to-date, otherwise we don't
            # need this check. (And the test will fail when we check for the empty dict
            # since vim gets upgraded in the install step.)
            if 'out-of-date' not in vim_info['status']:
                # Install a version of vim that should need upgrading
                ret = self.run_function('pkg.install', ['vim', vim_version])
                if not isinstance(ret, dict):
                    if ret.startswith('ERROR'):
                        self.skipTest('Could not install earlier vim to complete test.')
                else:
                    self.assertNotEqual(ret, {})

            # Run a system upgrade, which should catch the fact that Vim needs upgrading, and upgrade it.
            ret = self.run_function(func)

            # The changes dictionary should not be empty.
            if 'changes' in ret:
                self.assertIn('vim', ret['changes'])
            else:
                self.assertIn('vim', ret)
        else:
            ret = self.run_function('pkg.list_upgrades')
            if ret == '' or ret == {}:
                self.skipTest('No updates available for this machine.  Skipping pkg.upgrade test.')
            else:
                ret = self.run_function(func)

                if 'Problem encountered' in ret:
                    self.skipTest('A problem was encountered when running pkg.upgrade. This test is '
                                  'meant to catch no-ops or problems with the salt function itself, '
                                  'not problems with actual package installation. Skipping.')

                # The changes dictionary should not be empty.
                self.assertNotEqual(ret, {})
                if 'changes' in ret:
                    self.assertNotEqual(ret['changes'], {})


if __name__ == '__main__':
    from integration import run_tests
    run_tests(PkgModuleTest)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rahul Handay <rahulha@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import oracle
import os

# Globals
oracle.__salt__ = {}
oracle.cx_Oracle = object()


@skipIf(NO_MOCK, NO_MOCK_REASON)
class OracleTestCase(TestCase):
    '''
    Test cases for salt.modules.oracle
    '''
    def test_run_query(self):
        '''
        Test for Run SQL query and return result
        '''
        with patch.object(oracle, '_connect', MagicMock()) as mock_connect:
            mock_connect.cursor.execute.fetchall.return_value = True
            with patch.object(oracle, 'show_dbs', MagicMock()):
                self.assertTrue(oracle.run_query('db', 'query'))

    def test_show_dbs(self):
        '''
        Test for Show databases configuration from pillar. Filter by `*args`
        '''
        with patch.dict(oracle.__salt__, {'pillar.get':
                                          MagicMock(return_value='a')}):
            self.assertDictEqual(oracle.show_dbs('A', 'B'),
                                 {'A': 'a', 'B': 'a'})

            self.assertEqual(oracle.show_dbs(), 'a')

    def test_version(self):
        '''
        Test for Server Version (select banner  from v$version)
        '''
        with patch.dict(oracle.__salt__, {'pillar.get':
                                          MagicMock(return_value='a')}):
            with patch.object(oracle, 'run_query', return_value='A'):
                self.assertDictEqual(oracle.version(), {})

    def test_client_version(self):
        '''
        Test for Oracle Client Version
        '''
        with patch.object(oracle, 'cx_Oracle',
                          MagicMock(side_effect=MagicMock())):
            self.assertEqual(oracle.client_version(), '')

    def test_show_pillar(self):
        '''
        Test for Show Pillar segment oracle.*
        '''
        with patch.dict(oracle.__salt__, {'pillar.get':
                                          MagicMock(return_value='a')}):
            self.assertEqual(oracle.show_pillar('item'), 'a')

    def test_show_env(self):
        '''
        Test for Show Environment used by Oracle Client
        '''
        with patch.object(os, 'environ',
                          return_value={'PATH': 'PATH',
                                        'ORACLE_HOME': 'ORACLE_HOME',
                                        'TNS_ADMIN': 'TNS_ADMIN',
                                        'NLS_LANG': 'NLS_LANG'}):
            self.assertDictEqual(oracle.show_env(), {})


if __name__ == '__main__':
    from integration import run_tests
    run_tests(OracleTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    mock_open,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import data

# Globals
data.__grains__ = {}
data.__salt__ = {}
data.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DataTestCase(TestCase):
    '''
    Test cases for salt.modules.data
    '''
    # 'clear' function tests: 1

    @patch('os.remove', MagicMock(return_value=''))
    def test_clear(self):
        '''
        Test if it clear out all of the data in the minion datastore
        '''
        mock = MagicMock(return_value='')
        with patch.dict(data.__opts__, {'cachedir': mock}):
            self.assertTrue(data.clear())

    # 'load' function tests: 1

    @patch('salt.utils.fopen', MagicMock(return_value=True))
    @patch('salt.payload.Serial.load', MagicMock(return_value=True))
    def test_load(self):
        '''
        Test if it return all of the data in the minion datastore
        '''
        mock = MagicMock(return_value='/')
        with patch.dict(data.__opts__, {'cachedir': mock}):
            self.assertTrue(data.load())

    # 'dump' function tests: 3

    def test_dump(self):
        '''
        Test if it replace the entire datastore with a passed data structure
        '''
        mock = MagicMock(return_value='/')
        with patch.dict(data.__opts__, {'cachedir': mock}):
            with patch('salt.utils.fopen', mock_open()):
                self.assertTrue(data.dump('{"eggs": "spam"}'))

    @patch('ast.literal_eval', MagicMock(return_value=''))
    def test_dump_isinstance(self):
        '''
        Test if it replace the entire datastore with a passed data structure
        '''
        self.assertFalse(data.dump('salt'))

    def test_dump_ioerror(self):
        '''
        Test if it replace the entire datastore with a passed data structure
        '''
        mock = MagicMock(return_value='/')
        with patch.dict(data.__opts__, {'cachedir': mock}):
            mock = MagicMock(side_effect=IOError(''))
            with patch('salt.utils.fopen', mock):
                self.assertFalse(data.dump('{"eggs": "spam"}'))

    # 'update' function tests: 1

    @patch('salt.modules.data.load', MagicMock(return_value={}))
    @patch('salt.modules.data.dump', MagicMock(return_value=True))
    def test_update(self):
        '''
        Test if it update a key with a value in the minion datastore
        '''
        self.assertTrue(data.update('foo', 'salt'))

    # 'get' function tests: 2

    @patch('salt.modules.data.load', MagicMock(return_value={'salt': 'SALT'}))
    def test_get(self):
        '''
        Test if it gets a value from the minion datastore
        '''
        self.assertEqual(data.get('salt'), 'SALT')

    @patch('salt.modules.data.load',
           MagicMock(return_value={'salt': 'SALT', 'salt1': 'SALT1'}))
    def test_get_vals(self):
        '''
        Test if it gets values from the minion datastore
        '''
        self.assertEqual(data.get(['salt', 'salt1']), ['SALT', 'SALT1'])

    # 'cas' function tests: 1

    @patch('salt.modules.data.load',
           MagicMock(return_value={'salt': 'SALT', 'salt1': 'SALT1'}))
    def test_cas_not_load(self):
        '''
        Test if it check and set a value in the minion datastore
        '''
        self.assertFalse(data.cas('salt3', 'SALT', 'SALTSTACK'))

    @patch('salt.modules.data.load',
           MagicMock(return_value={'salt': 'SALT', 'salt1': 'SALT1'}))
    def test_cas_not_equal(self):
        '''
        Test if it check and set a value in the minion datastore
        '''
        self.assertFalse(data.cas('salt', 'SALT', 'SALTSTACK'))

    @patch('salt.modules.data.load',
           MagicMock(return_value={'salt': 'SALT', 'salt1': 'SALT1'}))
    @patch('salt.modules.data.dump',
           MagicMock(return_value=True))
    def test_cas(self):
        '''
        Test if it check and set a value in the minion datastore
        '''
        self.assertTrue(data.cas('salt', 'SALTSTACK', 'SALT'))


if __name__ == '__main__':
    from integration import run_tests
    run_tests(DataTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Alexander Schwartz <alexander.schwartz@gmx.net>`
'''

# Import python libs
from __future__ import absolute_import
import os

# Import Salt Testing libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import archive as archive
from salt.ext.six.moves import zip  # pylint: disable=import-error,redefined-builtin

# Globals
archive.__salt__ = {}
archive.__grains__ = {'os': 'FooOS!'}
archive.__opts__ = {'cachedir': '/tmp', 'test': False, 'hash_type': 'sha256'}
archive.__env__ = 'test'


def _isfile_side_effect(path):
    '''
    MagicMock side_effect for os.path.isfile(). We don't want to use dict.get
    here because we want the test to fail if there's a path we haven't
    accounted for, so that we can add it.

    NOTE: This may fall over on some platforms if /usr/bin/tar does not exist.
    If so, just add an entry in the dictionary for the path being used for tar.
    '''
    return {
        '/tmp/foo.tar.gz': True,
        '/tmp/out': False,
        '/usr/bin/tar': True,
        '/bin/tar': True,
        '/tmp/test_extracted_tar': False,
    }[path]


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ArchiveTestCase(TestCase):

    def setUp(self):
        super(ArchiveTestCase, self).setUp()

    def tearDown(self):
        super(ArchiveTestCase, self).tearDown()

    def test_extracted_tar(self):
        '''
        archive.extracted tar options
        '''

        source = '/tmp/foo.tar.gz'
        tmp_dir = '/tmp/test_extracted_tar'
        test_tar_opts = [
            '--no-anchored foo',
            'v -p --opt',
            '-v -p',
            '--long-opt -z',
            'z -v -weird-long-opt arg',
        ]
        ret_tar_opts = [
            ['tar', 'x', '--no-anchored', 'foo', '-f'],
            ['tar', 'xv', '-p', '--opt', '-f'],
            ['tar', 'x', '-v', '-p', '-f'],
            ['tar', 'x', '--long-opt', '-z', '-f'],
            ['tar', 'xz', '-v', '-weird-long-opt', 'arg', '-f'],
        ]

        mock_true = MagicMock(return_value=True)
        mock_false = MagicMock(return_value=False)
        ret = {'stdout': ['cheese', 'ham', 'saltines'], 'stderr': 'biscuits', 'retcode': '31337', 'pid': '1337'}
        mock_run = MagicMock(return_value=ret)
        mock_source_list = MagicMock(return_value=(source, None))
        state_single_mock = MagicMock(return_value={'local': {'result': True}})
        list_mock = MagicMock(return_value={
            'dirs': [],
            'files': ['cheese', 'saltines'],
            'links': ['ham'],
            'top_level_dirs': [],
            'top_level_files': ['cheese', 'saltines'],
            'top_level_links': ['ham'],
        })
        isfile_mock = MagicMock(side_effect=_isfile_side_effect)

        with patch.dict(archive.__opts__, {'test': False,
                                           'cachedir': tmp_dir}):
            with patch.dict(archive.__salt__, {'file.directory_exists': mock_false,
                                               'file.file_exists': mock_false,
                                               'state.single': state_single_mock,
                                               'file.makedirs': mock_true,
                                               'cmd.run_all': mock_run,
                                               'archive.list': list_mock,
                                               'file.source_list': mock_source_list}):
                with patch.object(os.path, 'isfile', isfile_mock):
                    for test_opts, ret_opts in zip(test_tar_opts, ret_tar_opts):
                        ret = archive.extracted(tmp_dir,
                                                source,
                                                options=test_opts,
                                                enforce_toplevel=False)
                        ret_opts.append(source)
                        mock_run.assert_called_with(ret_opts,
                                                    cwd=tmp_dir + os.sep,
                                                    python_shell=False)

    def test_tar_gnutar(self):
        '''
        Tests the call of extraction with gnutar
        '''
        gnutar = MagicMock(return_value='tar (GNU tar)')
        source = '/tmp/foo.tar.gz'
        mock_false = MagicMock(return_value=False)
        mock_true = MagicMock(return_value=True)
        state_single_mock = MagicMock(return_value={'local': {'result': True}})
        run_all = MagicMock(return_value={'retcode': 0, 'stdout': 'stdout', 'stderr': 'stderr'})
        mock_source_list = MagicMock(return_value=(source, None))
        list_mock = MagicMock(return_value={
            'dirs': [],
            'files': ['stdout'],
            'links': [],
            'top_level_dirs': [],
            'top_level_files': ['stdout'],
            'top_level_links': [],
        })
        isfile_mock = MagicMock(side_effect=_isfile_side_effect)

        with patch.dict(archive.__salt__, {'cmd.run': gnutar,
                                           'file.directory_exists': mock_false,
                                           'file.file_exists': mock_false,
                                           'state.single': state_single_mock,
                                           'file.makedirs': mock_true,
                                           'cmd.run_all': run_all,
                                           'archive.list': list_mock,
                                           'file.source_list': mock_source_list}):
            with patch.object(os.path, 'isfile', isfile_mock):
                ret = archive.extracted('/tmp/out',
                                        source,
                                        options='xvzf',
                                        enforce_toplevel=False,
                                        keep=True)
                self.assertEqual(ret['changes']['extracted_files'], 'stdout')

    def test_tar_bsdtar(self):
        '''
        Tests the call of extraction with bsdtar
        '''
        bsdtar = MagicMock(return_value='tar (bsdtar)')
        source = '/tmp/foo.tar.gz'
        mock_false = MagicMock(return_value=False)
        mock_true = MagicMock(return_value=True)
        state_single_mock = MagicMock(return_value={'local': {'result': True}})
        run_all = MagicMock(return_value={'retcode': 0, 'stdout': 'stdout', 'stderr': 'stderr'})
        mock_source_list = MagicMock(return_value=(source, None))
        list_mock = MagicMock(return_value={
            'dirs': [],
            'files': ['stderr'],
            'links': [],
            'top_level_dirs': [],
            'top_level_files': ['stderr'],
            'top_level_links': [],
        })
        isfile_mock = MagicMock(side_effect=_isfile_side_effect)

        with patch.dict(archive.__salt__, {'cmd.run': bsdtar,
                                           'file.directory_exists': mock_false,
                                           'file.file_exists': mock_false,
                                           'state.single': state_single_mock,
                                           'file.makedirs': mock_true,
                                           'cmd.run_all': run_all,
                                           'archive.list': list_mock,
                                           'file.source_list': mock_source_list}):
            with patch.object(os.path, 'isfile', isfile_mock):
                ret = archive.extracted('/tmp/out',
                                        source,
                                        options='xvzf',
                                        enforce_toplevel=False,
                                        keep=True)
                self.assertEqual(ret['changes']['extracted_files'], 'stderr')

if __name__ == '__main__':
    from integration import run_tests
    run_tests(ArchiveTestCase, needs_daemon=False)
